/* $Id: config.cc,v 1.1 2005/07/11 21:45:39 whess Exp $ */
/* Overlord Server for the Refefree Programming Contest Judging System
 * Copyright (C) 2005 Wolfgang Hess
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
/**
 * \file config.cc
 * \brief Functions parsing the configuration file.
 *
 * Here all functions to parse a configuration or backup file and to create
 * a server from that are defined. This file contains the helper struct
 * config_tree as well. The actual XML parsing is done by xmlParser. See
 * http://iridia.ulb.ac.be/~fvandenb/tools/xmlParser.html for details on this
 * (LGPLed) parser.
 */
#include <fstream>
#include <iostream>
#include <sstream>
#include <map>
#include <set>
#include <string>
#include <cctype>
#include <boost/shared_ptr.hpp>

#include "xmlParser.h"

#include "server.hh"
#include "exception.hh"

/**
 * \struct config_tree
 * \brief Data structure to parse a configuration file into
 *
 * This structure is a single node of the parse tree of a configuration
 * file. The constructor is called recursively to create the parse tree.
 *
 * A fatal_exception is thrown if errors occur.
 */
struct config_tree
{
	config_tree(XMLNode that);
	
	boost::shared_ptr<config_tree> remove_child(const std::string &id);
	void exists_child(const std::string &id) const;
	void unique_child(const std::string &id) const;
	void no_children() const;
	bool remove_keyword(const std::string &key);
	std::string single_keyword() const;
	void no_keywords() const;
	
	typedef std::multimap<std::string, boost::shared_ptr<config_tree> > child_type;	///< type of the member 'children'
	typedef std::multiset<std::string> keyword_type;	///< type of the member 'keywords'
	
	child_type children;	///< map containing the children of this node
	keyword_type keywords;	///< map containing the words stored at this node
	std::string name;	///< name of this node
};

/**
 * \brief Creates a subtree from a parsed XML node.
 *
 * \param that the XML node to create the subtree for
 */
config_tree::config_tree(XMLNode that)
	: children(), keywords(), name(that.getName() ? that.getName(): "")
{
	for(int i = 0; i != that.nElement(); ++i)
	{
		XMLNodeContents content = that.enumContents(i);
		if(content.type == eNodeChild)
		{
			boost::shared_ptr<config_tree> child(new config_tree(content.child));
			children.insert(std::make_pair(child->name, child));
		}
		else if(content.type == eNodeText)
		{
			keywords.insert(std::string(content.text));
		}
	}
}

/**
 * \brief Remove and return a complete subtree
 *
 * This function removes a child with a given name (the first one if more than
 * one exist) and returns it. If no child exists a null pointer is returned.
 *
 * \param id name of the child to return
 * \return the removed child
 */
boost::shared_ptr<config_tree> config_tree::remove_child(const std::string &id)
{
	child_type::iterator i = children.find(id);
	if(i == children.end()) return boost::shared_ptr<config_tree>();
	else
	{
		boost::shared_ptr<config_tree> node0 = i->second;
		children.erase(i);
		return node0;
	}
}

/**
 * \brief Test for existance of a child.
 *
 * This function throws a fatal_exception if no child of the given name
 * exists.
 *
 * \param id name of the children to look at
 *
 * \see fatal_exception
 */
void config_tree::exists_child(const std::string &id) const
{
	if(children.find(id) == children.end())
		throw fatal_exception("config: No '" + id + "' list found at '" + name + "'.");
}

/**
 * \brief Test for uniqueness of a child.
 *
 * This function throws a fatal_exception if more than one child of
 * the given name exist.
 *
 * \param id name of the children to look at
 *
 * \see fatal_exception
 */
void config_tree::unique_child(const std::string &id) const
{
	child_type::const_iterator i = children.find(id);
	if(i != children.end())
	{
		++i;		
		if(i != children.end() && i->first == id)
			throw fatal_exception("config: List '" + id + "' is not unique, defined at '" + name + "'.");
	}
}

/**
 * \brief Test for non-existance of children.
 *
 * This function throws a fatal_exception if a child exists.
 *
 * \see fatal_exception
 */
void config_tree::no_children() const
{
	if(!children.empty())
		throw fatal_exception("config: Unexpected list '" + children.begin()->first + "' at '" + name + "'.");
}

/**
 * \brief Remove a keyword
 *
 * This function removes a keyword with a given name (the first one if more
 * than one exist) and returns true. If no such keyword exists false is returned.
 *
 * \param key name of the keyword to remove
 * \return whether a keyword was removed
 */
bool config_tree::remove_keyword(const std::string &key)
{
	keyword_type::iterator i = keywords.find(key);
	if(i == keywords.end()) return false;
	else
	{
		keywords.erase(i);
		return true;
	}
}

/**
 * \brief Remove the unique keyword
 *
 * This function removes the unique keyword and returns it.
 * When no keyword exists, there are several keywords or there are
 * children, a fatal_exception is thrown.
 *
 * \return the unique keyword
 */
std::string config_tree::single_keyword() const
{
	if(keywords.size() != 1 || !children.empty())
	{
		std::ostringstream os;
		os << "config: A single keyword is expected at '" + name + "'.";
		throw fatal_exception(os.str());
	}
	
	return *keywords.begin();
}

/**
 * \brief Test for non-existance of keywords.
 *
 * This function throws a fatal_exception if a keyword exists.
 *
 * \see fatal_exception
 */
void config_tree::no_keywords() const
{
	if(!keywords.empty())
		throw fatal_exception("config: Unexpected keyword '" + *keywords.begin() + "' at '" + name + "'.");
}

/**
 * \brief Output the (parsed) configuration tree
 *
 * This function is intended for debug purposes.
 *
 * \param os stream to output to
 * \param c configuration tree to output
 * \return the output stream
 */
std::ostream &operator<<(std::ostream &os, const config_tree &c)
{
	bool first = true;
	os << "{";
	for(config_tree::child_type::const_iterator i = c.children.begin();
		i != c.children.end(); ++i)
	{
		if(first) first = false;
		else os << ", ";
		
		os << i->first << ": " << *i->second;
	}
	for(config_tree::keyword_type::const_iterator i = c.keywords.begin();
		i != c.keywords.end(); ++i)
	{
		if(first) first = false;
		else os << ", ";
		
		os << *i;
	}
	os << "}";
	return os;
}

/**
 * \brief Creates a configured server object
 *
 * This function creates a completely configured server object from a
 * given configuration file. It calls create_default_state(), reads the
 * configuration file and calls the configuration function accordingly.
 *
 * \param config the filename of the configuration file to use
 * \param open_ports whether to listen to the ports defined in the configuration
 * \return a configured server object
 *
 * \see create_default_state() and config_tree
 */
std::auto_ptr<server_interface> state_from_config(const std::string &config,
	bool open_ports)
{
	std::auto_ptr<server_interface> state;
	state = create_default_state();
	state->set_configfile(config);
	
	std::ifstream is(config.c_str());
	if(!is)
		throw fatal_exception("config: Unable to open configuration '"
			+ config + "'.");
	std::string cfgstring;
	{
		char ch;
		while(is.get(ch)) cfgstring += ch;
	}
	if(!is.eof())
		throw fatal_exception("config: Error reading the configuration '"
			+ config + "'.");
	
	XMLResults Results;
	XMLNode root = XMLNode::parseString(cfgstring.c_str(), 0, &Results);
	if(Results.error != eXMLErrorNone)
	{
		std::ostringstream ex;
		ex << "config: Error parsing the configuration '" << config << "' in line "
			<< Results.nLine << " at column " << Results.nColumn << "; reason is "
			<< XMLNode::getError(Results.error) << ".";
		throw fatal_exception(ex.str());
	}
	
	config_tree cfg0(root);
	cfg0.exists_child("xml");
	cfg0.unique_child("xml");
	boost::shared_ptr<config_tree> cfg1 = cfg0.remove_child("xml");
	cfg1->exists_child("server-configuration");
	cfg1->unique_child("server-configuration");
	boost::shared_ptr<config_tree> cfg2 = cfg1->remove_child("server-configuration");
	config_tree &cfg = *cfg2;
	
	// output the configuration tree for debugging:
	// std::cout << cfg << std::endl;

	if(!cfg.keywords.empty())
		throw fatal_exception("config: Internal error.");
	
	{	
		cfg.exists_child("contest");
		cfg.unique_child("contest");
		boost::shared_ptr<config_tree> cfg_contest = cfg.remove_child("contest");
		
		// contest name
		{
			std::string name("a contest");
			cfg_contest->unique_child("name");
			boost::shared_ptr<config_tree> cfg_name = cfg_contest->remove_child("name");
			if(cfg_name.get() != 0) name = cfg_name->single_keyword();
			state->set_description(name);
		}
		
		// contest duration
		{
			int duration = 300;	// 5 hours = 300 minutes
			cfg_contest->unique_child("duration");
			boost::shared_ptr<config_tree> cfg_dur = cfg_contest->remove_child("duration");

			if(cfg_dur.get() != 0)
			{
				std::string duration_string = cfg_dur->single_keyword();
				std::istringstream duration_stream(duration_string);
				if(!((duration_stream >> duration) && duration > 0))
					throw fatal_exception("config: Positive integer as contest duration expected at '" + cfg_dur->name + "'.");
			}
			
			state->set_duration(duration);
		}
		
		// scoreboard duration
		{
			int scoreboard_duration = 240;	// 4 hours = 240 minutes
			cfg_contest->unique_child("scoreboard");
			boost::shared_ptr<config_tree> cfg_dur = cfg_contest->remove_child("scoreboard");

			if(cfg_dur.get() != 0)
			{
				std::string duration_string = cfg_dur->single_keyword();
				std::istringstream duration_stream(duration_string);
				if(!((duration_stream >> scoreboard_duration) && scoreboard_duration >= 0))
					throw fatal_exception("config: Non-negative integer as scoreboard duration expected at '" + cfg_dur->name + "'.");
			}
			
			state->set_scoreboard_duration(scoreboard_duration);
		}
		
		// submission penalty
		{
			int penalty = 20;	// 20 minutes per wrong submission
			cfg_contest->unique_child("penalty");
			boost::shared_ptr<config_tree> cfg_pen = cfg_contest->remove_child("penalty");

			if(cfg_pen.get() != 0)
			{
				std::string penalty_string = cfg_pen->single_keyword();
				std::istringstream penalty_stream(penalty_string);
				if(!((penalty_stream >> penalty) && penalty >= 0))
					throw fatal_exception("config: Non-negative integer as submission penalty expected at '" + cfg_pen->name + "'.");
			}
			
			state->set_penalty(penalty);
		}
		
		// programming language configuration
		{
			bool language_exists = false;
		
			boost::shared_ptr<config_tree> cfg_lang;
			while((cfg_lang = cfg_contest->remove_child("language")).get() != 0)
			{
				language_exists = true;
				
				cfg_lang->exists_child("id");
				cfg_lang->unique_child("id");			
				boost::shared_ptr<config_tree> cfg_id
					= cfg_lang->remove_child("id");
				std::string id = cfg_id->single_keyword();
			
				cfg_lang->exists_child("name");
				cfg_lang->unique_child("name");			
				boost::shared_ptr<config_tree> cfg_name
					= cfg_lang->remove_child("name");
				std::string name = cfg_name->single_keyword();
			
				cfg_lang->no_keywords();
				cfg_lang->no_children();

				state->add_language(id, name);
			}
						
			if(!language_exists)	// use standard programming language set
			{
				state->add_language("c", "C");
				state->add_language("cc", "C++");
				state->add_language("java", "Java");
			}
		}
		
		// problem set configuration
		{
			cfg_contest->exists_child("problem");
			
			boost::shared_ptr<config_tree> cfg_problem;
			while((cfg_problem = cfg_contest->remove_child("problem")).get() != 0)
			{
				cfg_problem->exists_child("id");
				cfg_problem->unique_child("id");			
				boost::shared_ptr<config_tree> cfg_id
					= cfg_problem->remove_child("id");
				std::string id = cfg_id->single_keyword();
			
				cfg_problem->exists_child("name");
				cfg_problem->unique_child("name");			
				boost::shared_ptr<config_tree> cfg_name
					= cfg_problem->remove_child("name");
				std::string name = cfg_name->single_keyword();
			
				cfg_problem->no_keywords();
				cfg_problem->no_children();

				state->add_problem(id, name);
			}
		}
		
		cfg_contest->no_keywords();
		cfg_contest->no_children();
	}
	
	// read the contestant configuration
	{
		cfg.exists_child("contestant");
		boost::shared_ptr<config_tree> cfg_con;
		while((cfg_con = cfg.remove_child("contestant")).get() != 0)
		{
			std::string login, passwd, name;
			cfg_con->exists_child("login");
			cfg_con->unique_child("login");
			login = cfg_con->remove_child("login")->single_keyword();
			cfg_con->exists_child("password");
			cfg_con->unique_child("password");
			passwd = cfg_con->remove_child("password")->single_keyword();
			cfg_con->exists_child("name");
			cfg_con->unique_child("name");
			name = cfg_con->remove_child("name")->single_keyword();
			
			bool flag_hide = false;
			while(cfg_con->remove_child("hide")) flag_hide = true;
			
			cfg_con->no_keywords();
			cfg_con->no_children();
			
			state->add_contestant(login, passwd, name, flag_hide);
		}
	}
	
	// read the judge configuration
	{
		boost::shared_ptr<config_tree> cfg_judge;
		while((cfg_judge = cfg.remove_child("judge")).get() != 0)
		{
			std::string login, passwd;
			cfg_judge->exists_child("login");
			cfg_judge->unique_child("login");
			login = cfg_judge->remove_child("login")->single_keyword();
			cfg_judge->exists_child("password");
			cfg_judge->unique_child("password");
			passwd = cfg_judge->remove_child("password")->single_keyword();
			
			cfg_judge->no_keywords();
			cfg_judge->no_children();
			
			state->add_judge(login, passwd);
		}
	}

	// maximal message size
	{
		int max_message_size = 256;	// 256kB is standard
		cfg.unique_child("max-message-size");
		boost::shared_ptr<config_tree> cfg_mms = cfg.remove_child("max-message-size");

		if(cfg_mms.get() != 0)
		{
			std::string mms_string = cfg_mms->single_keyword();
			std::istringstream mms_stream(mms_string);
			if(!((mms_stream >> max_message_size) && max_message_size > 0))
				throw fatal_exception("config: Positive integer as maximal message size expected at '" + cfg_mms->name + "'.");
		}

		state->set_max_message_size(max_message_size);
	}
	
	// heartbeat interval
	{
		int heartbeat_interval = 1;	// 1 second per heartbeat
		cfg.unique_child("heartbeat-interval");
		boost::shared_ptr<config_tree> cfg_hi = cfg.remove_child("heartbeat-interval");

		if(cfg_hi.get() != 0)
		{
			std::string hi_string = cfg_hi->single_keyword();
			std::istringstream hi_stream(hi_string);
			if(!((hi_stream >> heartbeat_interval) && heartbeat_interval >= 1))
				throw fatal_exception("config: Positive integer as heartbeat interval expected at '" + cfg_hi->name + "'.");
		}

		state->set_heartbeat_interval(heartbeat_interval);
	}
	
	// connection timeout
	{
		int connection_timeout = 15;	// 15 heartbeats is standard
		cfg.unique_child("connection-timeout");
		boost::shared_ptr<config_tree> cfg_cto = cfg.remove_child("connection-timeout");

		if(cfg_cto.get() != 0)
		{
			std::string cto_string = cfg_cto->single_keyword();
			std::istringstream cto_stream(cto_string);
			if(!((cto_stream >> connection_timeout) && connection_timeout >= 0))
				throw fatal_exception("config: Non-negative integer as connection timeout expected at '" + cfg_cto->name + "'.");
		}

		state->set_connection_timeout(connection_timeout);
	}
		
	// auto-finish
	{
		bool auto_finish = true;	// auto-finish is on by default
		cfg.unique_child("auto-finish");
		boost::shared_ptr<config_tree> cfg_af = cfg.remove_child("auto-finish");

		if(cfg_af.get() != 0)
		{
			std::string af_string = cfg_af->single_keyword();
			std::istringstream af_stream(af_string);
			std::string value;
			if(!(af_stream >> value))
				throw fatal_exception("config: 0, 1, true, false, on or off for auto-finish expected at '" + cfg_af->name + "'.");
			if(value == "true") auto_finish = true;
			else if(value == "on") auto_finish = true;
			else if(value == "false") auto_finish = false;
			else if(value == "off") auto_finish = false;
			else
			{
				std::istringstream af_stream2(af_string);
				int value = 0;
				if(!(af_stream2 >> value) || (value != 0 && value != 1))
					throw fatal_exception("config: 0, 1, true, false, on or off for auto-finish expected at '" + cfg_af->name + "'.");
				auto_finish = (value == 1);
			}
		}

		state->set_auto_finish(bool(auto_finish));
	}
		
	// read the port configuration
	{
		bool listen_exists = false;
		
		boost::shared_ptr<config_tree> cfg_listen;
		while((cfg_listen = cfg.remove_child("listen")).get() != 0)
		{
			listen_exists = true;
			
			bool flag_contestants = false, flag_judges = false, flag_local = false;
			
			cfg_listen->exists_child("flags");
			
			boost::shared_ptr<config_tree> cfg_flags;
			while((cfg_flags = cfg_listen->remove_child("flags")).get() != 0)
			{
				while(cfg_flags->remove_child("contestants")) flag_contestants = true;
				while(cfg_flags->remove_child("judges")) flag_judges = true;
				while(cfg_flags->remove_child("local")) flag_local = true;
				
				cfg_flags->no_children();
				cfg_flags->no_keywords();
			}
			
			cfg_listen->exists_child("port");
			cfg_listen->unique_child("port");
			boost::shared_ptr<config_tree> cfg_port
				= cfg_listen->remove_child("port");
			
			std::string port_string = cfg_port->single_keyword();
			boost::uint16_t port;
			std::istringstream port_stream(port_string);
			if(!((port_stream >> port) && port > 0 && port - 1 < 65535))
				throw fatal_exception("config: Port number between 1 and 65535 expected at '" + cfg_port->name + "'.");
			
			cfg_listen->no_keywords();
			cfg_listen->no_children();
			
			if(open_ports) state->add_port(port, flag_contestants, flag_judges, flag_local);
		}
		
		if(!listen_exists && open_ports)
			state->add_port(27251, true, true, false);
	}
	
	// configuration should be completed
	cfg.no_children();
	
	return state;
}

/**
 * \brief Creates a server object from the backup file
 *
 * This function creates a completely configured server object from
 * the backup file 'backup.overlord'. It reads the backup file,
 * calls state_from_config() with the configuration filename saved therein
 * and calls server_interface::restore_backup().
 *
 * \param open_ports whether to listen to the ports defined in the configuration
 * \return a configured server object
 *
 * \see state_from_config() and server_interface::restore_backup()
 */
std::auto_ptr<server_interface> state_from_backup(bool open_ports)
{
	std::ifstream backup_stream("backup.overlord",
		std::ios_base::in | std::ios_base::binary);
	
	if(!backup_stream)
		throw fatal_exception("config: Could not open the backup file.");
	
	std::string comment, state_string, time_string; 
	
	if(!(getline(backup_stream, comment)
		&& getline(backup_stream, state_string)
		&& getline(backup_stream, time_string)))
		throw fatal_exception("config: Error reading the backup file.");
	
	int elapsed_seconds = 0;
	{
		std::istringstream is(time_string);
		if(!(is >> elapsed_seconds) || elapsed_seconds < 0)
			throw fatal_exception("config: Backup file contains an illegal time.");
	}
	
	std::string config;
	{
		char ch = 0;
		while(backup_stream.get(ch))
			config += ch;
	}
	
	std::auto_ptr<server_interface> state = state_from_config(config, open_ports);
	state->restore_backup(state_string, elapsed_seconds);
	
	return state;
}
