/* $Id: files.cc,v 1.1 2005/07/11 21:45:39 whess Exp $ */
/* Overlord Server for the Refefree Programming Contest Judging System
 * Copyright (C) 2005 Wolfgang Hess
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
/**
 * \file files.cc
 * \brief Functions implementing the file system structure.
 *
 * Here all functions handling the 'File System Structure' as
 * described in the protocol documentation are defined.
 */
#include <iostream>
#include <sstream>
#include <string>
#include <fstream>
#include <cerrno>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>

#include "files.hh"
#include "exception.hh"
#include "caps.hh"

mode_t filemode(S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH);	///< mode of files created by the server

namespace
{
	/**
	 * \brief Create a temporary with contents
	 *
	 * This function creates a temporary file, writes its contents and
	 * returns its name.
	 *
	 * \param content contents to write to the file
	 * \return the name of the temporary file
	 */
	std::string write_temporary(const std::string &content)
	{
		const size_t data_size = content.size();
		const char *data_ptr = content.c_str();
	
		char temp[12] = "temp.XXXXXX";
		int fd = mkstemp(temp);
		if(fd == -1) throw fatal_exception("filesystem: Unable to create temporary file. ["
			+ std::string(strerror(errno)) + "]");

		try
		{
			try
			{
				ssize_t result = write(fd, data_ptr, data_size);
				if(result == -1)
				{
					throw fatal_exception("filesystem: Error writing a temporary file. ["
						+ std::string(strerror(errno)) + "]");
				}
				else if(result != ssize_t(data_size))
				{
					throw fatal_exception("filesystem: Error writing a temporary file.");
				}
				if(fsync(fd) == -1)
				{
					throw fatal_exception("filesystem: Error synchronizing a temporary file. ["
						+ std::string(strerror(errno)) + "]");
				}
			}
			catch(...)
			{
				close(fd);
				throw;
			}

			if(close(fd) == -1)
			{
				throw fatal_exception("filesystem: Error closing a temporary file. ["
					+ std::string(strerror(errno)) + "]");
			}

			// set the file mode	
			if(chmod(temp, filemode) == -1)
			{
				throw fatal_exception("filesystem: Error chmod-ing a temporary file. ["
					+ std::string(strerror(errno)) + "]");
			}
		}
		catch(...)
		{
			unlink(temp);
			throw;
		}
		
		return std::string(temp);
	}

	/**
	 * \brief Write a file with contents
	 *
	 * This function creates a file with the given name and writes its contents.
	 * If a file already exists it is overwritten.
	 *
	 * \param filename name of the file
	 * \param content contents to write to the file
	 */
	void write_file(const std::string &filename, const std::string &content)
	{	
		{
			std::ofstream filestream(filename.c_str(),
				std::ios_base::out | std::ios_base::trunc | std::ios_base::binary);
			if(!filestream) throw fatal_exception("filesystem: Unable to create " + filename + ".");

			filestream << content;
			if(!filestream) throw fatal_exception("filesystem: Error writing " + filename + ".");

			filestream.flush();
			if(!filestream) throw fatal_exception("filesystem: Error synchronizing " + filename + ".");
			filestream.close();
			if(!filestream) throw fatal_exception("filesystem: Error closing " + filename + ".");
		}
		
		// set the file mode
		if(chmod(filename.c_str(), filemode) != 0) throw fatal_exception("filesystem: Error chmod-ing " + filename + ".");
	}
}

/**
 * \brief Create files for a new submission.
 *
 * This function handles submissions by creating the necessary
 * files for a new submission.
 *
 * \param snum the zero-based index of this submission
 * \param submitter the login of the submitting contestant
 * \param problem the problem id for which this is supposed to be a solution
 * \param language the language id for the language in which the code is written
 * \param source the solution itself
 * \param time the time in minutes since the contest started
 */
void handle_submission(int snum, const std::string &submitter,
	const std::string &problem, const std::string &language,
	const std::string &source, int time)
{
	// create #.submission as a temporary file
	std::ostringstream submission_data;
	submission_data
		<< submitter << "\n"
		<< problem << "\n"
		<< language << "\n"
		<< time << "\n";
	
	std::string temp = write_temporary(submission_data.str());
	
	// create #.source
	{
		std::ostringstream source_file;
		source_file << snum << ".source";
		write_file(source_file.str(), source);
	}
		
	// create #.judgement
	{
		std::ostringstream judgement_file;
		judgement_file << snum << ".judgement";
		write_file(judgement_file.str(), "\nnew\n\n");
	}
	
	// rename temporary to #.submission
	{
		std::ostringstream submission_fs;
		submission_fs << snum << ".submission";
		const std::string submission_fstr = submission_fs.str();
		const char *submission_file = submission_fstr.c_str();
		if(rename(temp.c_str(), submission_file) == -1)	// might be an error (or NFS?)
		{
			// stat() to be sure!
			struct stat buf;
			if(stat(temp.c_str(), &buf) != 0)
				throw fatal_exception("filesystem: Error moving a submission file.");
		}
	}	
}

/**
 * \brief Check existance of a submission.
 *
 * This function checks whether a submission exists by looking at the
 * n.submission file.
 *
 * \param snum the zero-based index of the submission to look at
 * \return true if the submission exists, false otherwise
 */
bool exists_submission(int snum)
{
	struct stat buf;

	std::ostringstream submission_fs;
	submission_fs << snum << ".submission";
	const std::string submission_fstr = submission_fs.str();
	const char *submission_file = submission_fstr.c_str();

	return stat(submission_file, &buf) == 0;
}

/**
 * \brief Get never changing information about a submission.
 *
 * This function accesses the information stored in the n.submission
 * file. The submission file must exist.
 *
 * \param snum the zero-based index of the submission to look at
 * \param submitter is set to the login of the contestant who submitted
 * \param stime is set to the submission time in minutes since contest start
 * \param problem is set to the problem id
 * \param language is set to the id of the used language
 */
void get_submission_info(int snum, std::string &submitter,
	int &stime, std::string &problem, std::string &language)
{
	std::ostringstream submission_fs;
	submission_fs << snum << ".submission";
	const std::string submission_str(submission_fs.str());
	const char * const submission_file = submission_str.c_str();

	std::ifstream submission_stream(submission_file,
		std::ios_base::in | std::ios_base::binary);
	if(!submission_stream)
		throw fatal_exception("filesystem: Error opening submission file "
			+ submission_str + ".");
	
	std::string submission_data;
	{
		char ch;
		while(submission_stream.get(ch)) submission_data += ch;
	}
	if(submission_stream.bad())
		throw fatal_exception("filesystem: Error reading submission file "
			+ submission_str + ".");
	
	std::istringstream is(submission_data);
	if(!getline(is, submitter) ||
		!getline(is, problem) ||
		!getline(is, language) ||
		!(is >> stime))
		throw fatal_exception("filesystem: Malformed submission file "
			+ submission_str + ".");
}

/**
 * \brief Get changing judging information about a submission.
 *
 * This function accesses the judgement information stored in the
 * n.judgement and files linked to it. The file suffix must be given
 * to this function and should be '.judgement', '.notified' or
 * '.jnotified'. The corresponding file must exist unless CAP_NOTIFIES
 * is defined. In this case a new result is returned if the file
 * does not exist.
 *
 * \param snum the zero-based index of the submission to look at
 * \param state is set to the state as stored in the file (usually new,
 *     unjudged, accepted, rejected or ignored)
 * \param reason is set to the reason given in the file
 * \param judge is set to the judge id given in the file
 * \param lock is set to 'locked' if the lock (notify) file exists
 * \param suffix the suffix of the file to look at
 *
 * \see new_submission_state() and CAP_NOTIFIES
 */
void get_submission_state(int snum, std::string &state,
	std::string &reason, std::string &judge, std::string &lock,
	const char *suffix)
{
	state = "new";
	reason = "";
	judge = "";
	lock = "";
	
	std::ostringstream judgement_fs;
	judgement_fs << snum << suffix;
	const std::string judgement_str(judgement_fs.str());
	const char * const judgement_file = judgement_str.c_str();
	std::ifstream judgement_stream(judgement_file,
		std::ios_base::in | std::ios_base::binary);
	if(judgement_stream)
	{
		std::string judge2, verdict, reason2;
		if(!getline(judgement_stream, judge2) ||
			!getline(judgement_stream, verdict) ||
			!getline(judgement_stream, reason2))
		{
			std::cout << "filesystem: Malformed judgement " << judgement_str
				<< " ignored." << std::endl;
		}
		else
		{
			state = verdict;
			reason = reason2;
			judge = judge2;
			
			std::ostringstream lock_fs;
			lock_fs << snum
#ifdef CAP_NOTIFIES
				<< ".lock.notified";
#else
				<< ".lock";
#endif
			const std::string lock_str(lock_fs.str());
			const char * const lock_file = lock_str.c_str();
			struct stat buf;
			// look for the #.lock file
			lock = (stat(lock_file, &buf) == 0) ? "locked" : "";				
		}
	}
#ifndef CAP_NOTIFIES
	else throw fatal_exception("filesystem: Missing file " + judgement_str + ".");
#endif
}

/**
 * \brief Get source code of a submission.
 *
 * This function accesses the source code stored in the
 * n.source file. The submission must exist.
 *
 * \param snum the zero-based index of the submission to look at
 * \param source is set to the source code read from the file
 */
void get_submission_source(int snum, std::string &source)
{
	std::ostringstream source_fs;
	source_fs << snum << ".source";
	const std::string source_fstr = source_fs.str();
	const char *source_file = source_fstr.c_str();
	
	std::ifstream source_stream(source_file,
		std::ios_base::in | std::ios_base::binary);
	if(!source_stream)
		throw fatal_exception("filesystem: Error opening source file "
			+ source_fstr + ".");
	
	source.clear();
	{
		char ch;
		while(source_stream.get(ch)) source += ch;
	}
	
	if(source_stream.bad())
		throw fatal_exception("filesystem: Error reading source file "
			+ source_fstr + ".");
}

/**
 * \brief Checks whether a notification should be sent.
 *
 * This function checks whether a notification should be sent and if so
 * links the corresponding n.jnotified or n.notified file. When judge_queries
 * is set, notification of the lock state is checked as well. The submission
 * must exist.
 *
 * If CAP_NOTIFIES is not defined the function is disabled and
 * does nothing but return false. It is not called in this case anyway.
 *
 * \param snum the zero-based index of the submission to look at
 * \param judge_queries true for checking n.jnotified, or
 *     false for checking n.notified
 * \return true if the notification file was linked, false otherwise
 *
 * \see CAP_NOTIFIES
 */
bool need_submission_notification(int snum, bool judge_queries)
{
#ifdef CAP_NOTIFIES
	bool lock_changed = false;
	if(judge_queries)
	{
		std::ostringstream lock_fs;
		lock_fs << snum << ".lock";
		const std::string lock_str(lock_fs.str());
		const char * const lock_file = lock_str.c_str();
		std::ostringstream lock_notified_fs;
		lock_notified_fs << snum << ".lock.notified";
		const std::string lock_notified_str(lock_notified_fs.str());
		const char * const lock_notified_file = lock_notified_str.c_str();
		struct stat buf;
		
		// look for the #.lock and #.lock.notified file
		bool lock_there = (stat(lock_file, &buf) == 0);
		bool notify_there = (stat(lock_notified_file, &buf) == 0);
		if(lock_there != notify_there)	// lock state has changed
		{
			lock_changed = true;
			
			if(!lock_there)
				unlink(lock_notified_file);
			else
				link(lock_file, lock_notified_file);
		}
	}
	
	std::ostringstream judgement_fs;
	judgement_fs << snum << ".judgement";
	const std::string judgement_str(judgement_fs.str());
	const char * const judgement_file = judgement_str.c_str();
	struct stat buf;
	if(stat(judgement_file, &buf) == 0)	// look for the #.judgement file
	{
		if(buf.st_nlink == 3 ||	// all notifications already sent
			(buf.st_nlink == 2 && judge_queries))	// judge notification already sent
		{
			if(judge_queries && lock_changed) return true;
			else return false;
		}
		else if(buf.st_nlink == 1 && !judge_queries) return false;	// judge notification must be sent first
		else if(buf.st_nlink > 3) throw fatal_exception("filesystem: nlink too large stat-ing " + judgement_str + ".");
	}
	else throw fatal_exception("filesystem: " + judgement_str + " missing.");
	
	std::ostringstream notified_fs;
	notified_fs << snum << (judge_queries ? ".jnotified" : ".notified");
	const std::string notified_str(notified_fs.str());
	const char * const notified_file = notified_str.c_str();
	unlink(notified_file);	// unlink first
	if(stat(notified_file, &buf) == 0)	// notified still exists?
		throw fatal_exception("filesystem: Error unlinking " + notified_str + ".");

	// link to the new judgement
	if(judge_queries)
	{
		if(link(judgement_file, notified_file) == 0)
			return true;	// success
	}
	else
	{
		std::ostringstream jnotified_fs;
		jnotified_fs << snum << ".jnotified";
		const std::string jnotified_str(jnotified_fs.str());
		const char * const jnotified_file = jnotified_str.c_str();
		if(stat(jnotified_file, &buf) != 0)
			return false;	// seems to be the result of 2 hard links while renaming
		if(buf.st_nlink == 1)
			return false;	// seems to be outdated
		if(link(jnotified_file, notified_file) == 0)
			return true;	// success
	}
		
		
	if(stat(notified_file, &buf) != 0)	// notified not linked?
		throw fatal_exception("filesystem: Error linking " + notified_str + ".");
	
	return true;
#else
	return false;
#endif
}

/**
 * \brief Creates a new judgement.
 *
 * This function atomically creates a new judgement file.
 *
 * \param snum the zero-based index of the submission to look at
 * \param state the state to store in the file  (usually new,
 *     unjudged, accepted, rejected or ignored)
 * \param reason the reason to store in the file
 * \param judge the judge id to store in the file
 */
void new_submission_state(int snum, const std::string &state,
	const std::string &reason, const std::string &judge)
{
	// create #.judgement as a temporary file
	std::ostringstream state_data;
	state_data
		<< judge << "\n"
		<< state << "\n"
		<< reason << "\n";
	std::string temp = write_temporary(state_data.str());
	
	// rename temporary to #.judgement
	{
		std::ostringstream judgement_fs;
		judgement_fs << snum << ".judgement";
		const std::string judgement_fstr = judgement_fs.str();
		const char *judgement_file = judgement_fstr.c_str();
		if(rename(temp.c_str(), judgement_file) == -1)	// might be an error (or NFS?)
		{
			// stat() to be sure!
			struct stat buf;
			if(stat(temp.c_str(), &buf) != 0)
				throw fatal_exception("filesystem: Error moving a judgement file.");
		}
	}
}

/**
 * \brief Tries to get ownership of a lock file.
 *
 * This function tries to link a file, thus getting ownership
 * of the filename atomically, or failing if it already exists.
 *
 * \param lockname the name to which to link
 * \return true if the function successfully linked the name, false if it failed
 */
bool get_lock_file(const char *lockname)
{
	std::string temp = write_temporary(std::string());
		
	bool retval = true;	// assume that we get the lock, change otherwise
	struct stat buf;
	if(link(temp.c_str(), lockname) == -1)	// might be an error (or NFS?)
	{
		// stat() to be sure!
		if(stat(temp.c_str(), &buf) != 0)
			throw fatal_exception("lock: Error linking a lock file.");
			
		if(buf.st_nlink == 1)	// lock is already taken
			retval = false;	// lock _not_ taken by _us_
		else if(buf.st_nlink != 2)	// huh?
 			throw fatal_exception("lock: nlink too large stat-ing temporary "
				+ temp + ".");
	}
	
	unlink(temp.c_str());	// unlink
	if(stat(temp.c_str(), &buf) == 0)	// temporary still exists?
		throw fatal_exception(std::string("lock: Error unlinking temporary ")
			+ temp + ".");
	
	return retval;
}

/**
 * \brief Remove a lock file.
 *
 * This function unlinks a lock file, thus releasing the lock.
 *
 * \param lockname the name which to unlink
 */
void release_lock_file(const char *lockname)
{	
	unlink(lockname);	// unlink
	// we cannot check if we succeeded, since the lock might be taken again
	// in the meanwhile...
}

/**
 * \brief Rewrite 'backup.overlord' atomically.
 *
 * This function atomically rewrites 'backup.overlord'. It does not throw exceptions.
 *
 * \param backup_data_string the contents to write in the new backup file
 * \return true on success, false on failure
 */
bool save_server_state_backup(const std::string &backup_data_string)
{
	try
	{
		// create backup file as a temporary file first
		std::string temp = write_temporary(backup_data_string);

		// rename temporary to backup.overlord
		{
			if(rename(temp.c_str(), "backup.overlord") == -1)	// might be an error (or NFS?)
			{
				// stat() to be sure!
				struct stat buf;
				if(stat(temp.c_str(), &buf))
				{
					std::cout << "filesystem: Error moving the server backup file." << std::endl;
					return false;
				}
			}
		}
	
		return true;
	}
	catch(const fatal_exception &e)
	{
		std::cout << e.what() << std::endl
			<< "filesystem: Unable to write server backup." << std::endl;
		return false;
	}
}

/**
 * \brief Create files for a new clarification request.
 *
 * This function handles clarification requests by creating the necessary
 * files for a new request.
 *
 * If CAP_CLARIFICATIONS is not defined this function does nothing. Neither
 * is it called in this case.
 *
 * \param cnum the zero-based index of this request
 * \param submitter the login of the submitting client
 * \param problem the problem id for which problem this is a request, or empty if
 *     it is a general question.
 * \param question the request itself
 * \param time the time in minutes since the contest started
 *
 * \see CAP_CLARIFICATIONS
 */
void handle_clarification(int cnum, const std::string &submitter,
	const std::string &problem, const std::string &question, int time)
{
#ifdef CAP_CLARIFICATIONS
	// create C#.clarification as a temporary file
	std::ostringstream clarification_data;
	clarification_data
		<< submitter << "\n"
		<< problem << "\n"
		<< time << "\n"
		<< question;

	std::string temp = write_temporary(clarification_data.str());	

	// create C#.answer
	{
		std::ostringstream answer_file;
		answer_file << "C" << cnum << ".answer";
		write_file(answer_file.str(), "\nunanswered\n\n");
	}
	
	// rename temporary to C#.clarification
	{
		std::ostringstream clarification_fs;
		clarification_fs << "C" << cnum << ".clarification";
		const std::string clarification_fstr = clarification_fs.str();
		const char *clarification_file = clarification_fstr.c_str();
		if(rename(temp.c_str(), clarification_file) == -1)	// might be an error (or NFS?)
		{
			// stat() to be sure!
			struct stat buf;
			if(stat(temp.c_str(), &buf) != 0)
				throw fatal_exception("filesystem: Error moving a clarification file.");
		}
	}	
#endif
}

/**
 * \brief Check existance of a clarification request.
 *
 * This function checks whether a request exists by looking at the
 * Cn.clarification file.
 *
 * If CAP_CLARIFICATIONS is not defined this function does nothing and
 * returns false. It is not called in this case.
 *
 * \param cnum the zero-based index of the request to look at
 * \return true if the request exists, false otherwise
 *
 * \see CAP_CLARIFICATIONS
 */
bool exists_clarification(int cnum)
{
#ifdef CAP_CLARIFICATIONS
	struct stat buf;

	std::ostringstream clarification_fs;
	clarification_fs << "C" << cnum << ".clarification";
	const std::string clarification_fstr = clarification_fs.str();
	const char *clarification_file = clarification_fstr.c_str();

	return stat(clarification_file, &buf) == 0;
#else
	return false;
#endif
}

/**
 * \brief Get never changing information about a clarification request.
 *
 * This function accesses the information stored in the Cn.clarification
 * file. The request must exist.
 *
 * If CAP_CLARIFICATIONS is not defined this function does nothing. Neither
 * is it called in this case.
 *
 * \param cnum the zero-based index of the request to look at
 * \param submitter is set to the login of the client who submitted the request
 * \param problem is set to the problem id
 * \param stime is set to the submission time of the request in minutes since contest start
 * \param question is set to the question (i.e. the clarification request itself)
 *
 * \see CAP_CLARIFICATIONS
 */
void get_clarification_info(int cnum, std::string &submitter,
	std::string &problem, int &stime, std::string &question)
{
#ifdef CAP_CLARIFICATIONS
	std::ostringstream clarification_fs;
	clarification_fs << "C" << cnum << ".clarification";
	const std::string clarification_str(clarification_fs.str());
	const char * const clarification_file = clarification_str.c_str();

	std::ifstream clarification_stream(clarification_file,
		std::ios_base::in | std::ios_base::binary);
	if(!clarification_stream)
		throw fatal_exception("filesystem: Error opening clarification file "
			+ clarification_str + ".");
	
	std::string time_str;
	if(!getline(clarification_stream, submitter) ||
		!getline(clarification_stream, problem) ||
		!getline(clarification_stream, time_str))
		throw fatal_exception("filesystem: Malformed clarification file "
			+ clarification_str + ".");	
	{
		// read the time
		std::istringstream time_stream(time_str);
		if(!(time_stream >> stime))
			throw fatal_exception("filesystem: Malformed time in clarification file "
				+ clarification_str + ".");			
	}

	question.clear();
	{
		char ch;
		while(clarification_stream.get(ch)) question += ch;
	}
	
	if(clarification_stream.bad())
		throw fatal_exception("filesystem: Error reading clarification file "
			+ clarification_str + ".");	
#endif
}

/**
 * \brief Get changing clarification information.
 *
 * This function accesses the clarification request information stored in the
 * Cn.answer and files linked to it. The file suffix must be given
 * to this function and should be '.answer', '.notified' or '.onotified'.
 * The corresponding file must exist unless CAP_NOTIFIES is defined.
 * In this case a unanswered result is returned if the file does not exist.
 *
 * If CAP_CLARIFICATIONS is not defined this function does nothing. Neither
 * is it called in this case.
 *
 * \param cnum the zero-based index of the request to look at
 * \param state is set to the state as stored in the file (usually unanswered,
 *     questioner or everyone)
 * \param ctime is set to the last time in minutes when the state changed, empty if never
 * \param judge is set to the judge id given in the file
 * \param lock is set to 'locked' if the lock (notify) file exists
 * \param answer is set to the answer given in the file
 * \param suffix the suffix of the file to look at
 *
 * \see new_clarification_state(), CAP_NOTIFIES and CAP_CLARIFICATIONS
 */
void get_clarification_state(int cnum, std::string &state,
	std::string &ctime, std::string &judge, std::string &lock,
	std::string &answer, const char *suffix)
{
#ifdef CAP_CLARIFICATIONS
	state = "unanswered";
	ctime = "";
	judge = "";
	lock = "";
	answer = "";
	
	std::ostringstream answer_fs;
	answer_fs << "C" << cnum << suffix;
	const std::string answer_str(answer_fs.str());
	const char * const answer_file = answer_str.c_str();
	std::ifstream answer_stream(answer_file,
		std::ios_base::in | std::ios_base::binary);
	if(answer_stream)
	{
		std::string judge2, state2, ctime2;
		if(!getline(answer_stream, judge2) ||
			!getline(answer_stream, state2) ||
			!getline(answer_stream, ctime2))
		{
			std::cout << "filesystem: Malformed clarification answer " << answer_str
				<< " ignored." << std::endl;
		}
		else
		{
			std::string answer2;
			{
				char ch;
				while(answer_stream.get(ch)) answer2 += ch;
			}		
			
			if(answer_stream.bad())
			{
				std::cout << "filesystem: Error reading clarification answer "
					<< answer_str << " ignored.";	
			}
			else
			{
				state = state2;
				ctime = ctime2;
				judge = judge2;
				answer = answer2;
				
				std::ostringstream lock_fs;
				lock_fs << "C" << cnum
#ifdef CAP_NOTIFIES
					<< ".lock.notified";
#else
					<< ".lock";
#endif
				const std::string lock_str(lock_fs.str());
				const char * const lock_file = lock_str.c_str();
				struct stat buf;
				// look for the C#.lock file
				lock = (stat(lock_file, &buf) == 0) ? "locked" : "";				
			}
		}
	}
#ifndef CAP_NOTIFIES
	else throw fatal_exception("filesystem: Missing file " + answer_str + ".");
#endif

#endif
}

/**
 * \brief Checks whether a clarification notification should be sent.
 *
 * This function checks whether a notification should be sent and if so
 * links the corresponding Cn.onotified or Cn.notified file. When other_queries
 * is set, notification of the lock state is checked as well. The request
 * must exist. 
 *
 * If CAP_NOTIFIES or CAP_CLARIFICATIONS is not defined the function is
 * disabled and does nothing but return false. It is not called in this
 * case anyway.
 *
 * \param cnum the zero-based index of the request to look at
 * \param state_change is set to true when the state changed, not only
 *     the lock
 * \param other_queries true for checking Cn.onotified, or
 *     false for checking Cn.notified
 * \return true if the notification file was linked, false otherwise
 *
 * \see CAP_NOTIFIES and CAP_CLARIFICATIONS
 */
bool need_clarification_notification(int cnum, bool &state_change,
	bool other_queries)
{
#ifdef CAP_CLARIFICATIONS
#ifdef CAP_NOTIFIES
	bool lock_changed = false;
	state_change = false;
	if(other_queries)
	{
		std::ostringstream lock_fs;
		lock_fs << "C" << cnum << ".lock";
		const std::string lock_str(lock_fs.str());
		const char * const lock_file = lock_str.c_str();
		std::ostringstream lock_notified_fs;
		lock_notified_fs << "C" << cnum << ".lock.notified";
		const std::string lock_notified_str(lock_notified_fs.str());
		const char * const lock_notified_file = lock_notified_str.c_str();
		struct stat buf;
		
		// look for the C#.lock and C#.lock.notified file
		bool lock_there = (stat(lock_file, &buf) == 0);
		bool notify_there = (stat(lock_notified_file, &buf) == 0);
		if(lock_there != notify_there)	// lock state has changed
		{
			lock_changed = true;
			
			if(!lock_there)
				unlink(lock_notified_file);
			else
				link(lock_file, lock_notified_file);
		}
	}

	std::ostringstream answer_fs;
	answer_fs << "C" << cnum << ".answer";
	const std::string answer_str(answer_fs.str());
	const char * const answer_file = answer_str.c_str();
	struct stat buf;
	if(stat(answer_file, &buf) == 0)	// look for the C#.answer file
	{
		if(buf.st_nlink == 3 ||	// all notifications already sent
			(buf.st_nlink == 2 && other_queries))	// other notification already sent
		{
			if(other_queries && lock_changed) return true;
			else return false;
		}
		else if(buf.st_nlink == 1 && !other_queries) return false;	// other notification must be sent first
		else if(buf.st_nlink > 3) throw fatal_exception("filesystem: nlink too large stat-ing " + answer_str + ".");
	}
	else throw fatal_exception("filesystem: " + answer_str + " missing.");
	
	state_change = true;
	
	std::ostringstream notified_fs;
	notified_fs << "C" << cnum << (other_queries ? ".onotified" : ".notified");
	const std::string notified_str(notified_fs.str());
	const char * const notified_file = notified_str.c_str();
	unlink(notified_file);	// unlink first
	if(stat(notified_file, &buf) == 0)	// notified still exists?
		throw fatal_exception("filesystem: Error unlinking " + notified_str + ".");

	// link to the new answer
	if(other_queries)
	{
		if(link(answer_file, notified_file) == 0)
			return true;	// success
	}
	else
	{
		std::ostringstream onotified_fs;
		onotified_fs << "C" << cnum << ".onotified";
		const std::string onotified_str(onotified_fs.str());
		const char * const onotified_file = onotified_str.c_str();
		if(stat(onotified_file, &buf) != 0)
			return false;	// seems to be the result of 2 hard links while renaming
		if(buf.st_nlink == 1)
			return false;	// seems to be outdated
		if(link(onotified_file, notified_file) == 0)
			return true;	// success
	}
		
	if(stat(notified_file, &buf) != 0)	// notified not linked?
		throw fatal_exception("filesystem: Error linking " + notified_str + ".");
	
	return true;
#else
	return false;
#endif
#else
	return false;
#endif
}

/**
 * \brief Creates a new answer.
 *
 * This function atomically creates a new answer file.
 *
 * If CAP_CLARIFICATIONS is not defined this function does nothing. Neither
 * is it called in this case.
 *
 * \param cnum the zero-based index of the request to look at
 * \param state the state to store in the file  (usually unanswered,
 *     questioner or everyone)
 * \param ctime the current time in minutes to store in the file
 * \param judge the judge id to store in the file
 * \param answer the answer to store in the file
 *
 * \see CAP_CLARIFICATIONS
 */
void new_clarification_state(int cnum, const std::string &state,
	int ctime, const std::string &judge, const std::string &answer)
{
#ifdef CAP_CLARIFICATIONS
	// create #.answer as a temporary file
	std::ostringstream state_data;
	state_data
		<< judge << "\n"
		<< state << "\n"
		<< ctime << "\n"
		<< answer;
	std::string temp = write_temporary(state_data.str());
	
	// rename temporary to #.answer
	{
		std::ostringstream answer_fs;
		answer_fs << "C" << cnum << ".answer";
		const std::string answer_fstr = answer_fs.str();
		const char *answer_file = answer_fstr.c_str();
		if(rename(temp.c_str(), answer_file) == -1)	// might be an error (or NFS?)
		{
			// stat() to be sure!
			struct stat buf;
			if(stat(temp.c_str(), &buf) != 0)
				throw fatal_exception("filesystem: Error moving an answer file.");
		}
	}
#endif
}

/**
 * \brief Rewrite 'scoreboard.xml' atomically.
 *
 * This function atomically rewrites 'scoreboard.xml'. It does not throw exceptions.
 *
 * \param scoreboard_string the contents to write in the new scoreboard file
 * \return true on success, false on failure
 */
bool update_scoreboard_file(const std::string &scoreboard_string)
{
	try
	{
		// create scoreboard file as a temporary file first
		std::string temp = write_temporary(scoreboard_string);

		// rename temporary to scoreboard.xml
		{
			if(rename(temp.c_str(), "scoreboard.xml") == -1)	// might be an error (or NFS?)
			{
				// stat() to be sure!
				struct stat buf;
				if(stat(temp.c_str(), &buf))
				{
					std::cout << "filesystem: Error moving the scoreboard file." << std::endl;
					return false;
				}
			}
		}
	
		return true;
	}
	catch(const fatal_exception &e)
	{
		std::cout << e.what() << std::endl
			<< "filesystem: Unable to write scoreboard file." << std::endl;
		return false;
	}
}
