/* $Id: main.cc,v 1.2 2005/07/19 16:34:21 whess Exp $ */
/* Overlord Server for the Refefree Programming Contest Judging System
 * Copyright (C) 2005 Wolfgang Hess
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
/**
 * \file main.cc
 * \brief This file contains the function main.
 *
 * Here the program arguments are parsed and the server get started if
 * appropriate.
 */

/**
 * \mainpage
 *
 * \section overview_sec Overview
 *
 * This is the source code of the Overlord server belonging to the Refefree
 * online judging system for programming contests. This program is free
 * software licensed under the terms of the GPL. This server listens for
 * connections of clients which can be either contestants or judges.
 * Contestants can send in submissions and clarifications and judges can
 * judge or answer these. The server itself does not execute submissions or
 * perform actual judging, it rather distributes information to the judges
 * and the answers to the correct client and stores information about the
 * course of the contest.
 *
 * The server is written with dependability in mind. Thus the code is kept
 * simple, aims to be secure and robust. Much of the contest state is written
 * to files in a human-readable format to be able to examine the contest
 * state in case the server fails anyway.
 *
 * The implemented protocol is described in the 'protocol.pdf' file. See
 * this file for details on the protocol and the files the server creates
 * when it is running (containing submissions, judgements, clarification
 * requests and answers, implementing locking). Communications is done by
 * plain TCP sockets. Connections are established by the clients which are
 * welcomed by the server directly after connection setup. Each client
 * communicates with the server using a single connection.
 *
 * The compile-time configuration is contained in caps.hh, where clarifications,
 * automatical configuration of the clients over the network, the XML
 * scoreboard, notification, etc. can be enabled and disabled.
 *
 * The run-time configuration file is a UTF-8 XML file. See the sample
 * configuration named 'configuration.xml' for details. All possible
 * options are described in this file. For parsing an XML parser is used.
 *
 * The server as such is implemented in internal_state and is single-threaded.
 * Objects of this type are created using create_default_state() and must be
 * configured before they can be executed by calling
 * server_interface::execute().
 *
 * The program begins by executing main.cc, where the program arguments
 * are processed. Then functions from config.cc are called to create a
 * configured server object which is then executed. All further actions
 * are done from within the event loop in the server object.
 *
 * This event loop processes all active connections and calls several
 * notification functions every second, since the file system changes
 * are polled: internal_state::notify_judges(),
 * internal_state::notify_contestants(),
 * internal_state::notify_clarifications_others() and
 * internal_state::notify_clarifications_questioner().
 *
 * Messages received over the network are answered synchronously inside
 * the event loop.
 *
 * When the scoreboard is enabled, every minute a current scoreboard is
 * computed and saved to the file scoreboard.xml. This scoreboard is
 * used to answer the scoreboard requests and is updated until the scoreboard
 * duration is over.
 *
 * For more details have a look at the internal_state class.
 *
 * \section platforms_sec Platforms
 *
 * The server is written to work on POSIX systems. It uses the Boost library
 * for shared pointers, portability and safety and an LGPLed XML parser to
 * parse the configuration. Both are included at least in a form that is
 * usable to compile the source code, so no external libraries beside the
 * standard C++ library are necessary. 
 *
 * The following platforms are known to work.
 *
 * \subsection os_subsec Operating Systems
 * - Linux 2.6.5, 2.6.10
 * - Solaris 8, 9 (aka SunOS 5.8, 5.9)
 * - MacOS X 10.2.8
 *
 * \subsection cc_subsec C++ Compilers
 * - GCC 3.2, 3.3.3, 3.3.5, 3.4.1
 *
 * \section tests_sec Tests
 *
 * Nothing to see here so far.
 */
#include <iostream>
#include <vector>
#include <iomanip>
#include <sstream>
#include <exception>

#include "exception.hh"
#include "server.hh"
#include "caps.hh"

/**
 * \brief Writes usage information to the standard output.
 *
 * The function lists all options.
 * 
 * \return zero
 */
int usage()
{
	std::cout
		<< "Usage: overlord [options]\nOptions:\n"
		
		<< std::setw(30) << std::left << "  -h, --help" << "Print this message and exit.\n"
		<< std::setw(30) << std::left << "  -v, --version" << "Print the server version.\n"
		<< std::setw(30) << std::left << "  -c, --config CONFIG" << "Start using configuration CONFIG.\n"
		<< std::setw(30) << std::left << "  -r, --restore" << "Restore state from the backup.overlord file.\n"
#ifdef CAP_SCOREBOARD
		<< std::setw(30) << std::left << "  -s, --scoreboard" << "Write a current scoreboard.xml file.\n"
#endif
		
		<< std::endl;
	return 0;
}

/**
 * \brief Writes version information to the standard output.
 *
 * The function writes the version as found in caps.hh and a
 * copyright information.
 * 
 * \return zero
 *
 * \see CAP_VERSION
 */
int version()
{
	std::cout
		<< CAP_FULLVERSION << "\n"
		<< "Copyright (C) 2005 Wolfgang Hess\n"
		<< "This is free software; see the source for copying conditions.  There is NO\n"
		<< "warranty; not even for MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.\n"
		<< "Build " << __DATE__ << " " << __TIME__ << "\n"
		<< std::endl;
	return 0;
}

/**
 * \brief The more C++-like main function.
 *
 * This function parses the program arguments, shows usage or version
 * information, or starts a server either from a configuration or backup.
 *
 * \return value intended for main()
 *
 * \see usage(), version(), state_from_config() and state_from_backup()
 */
int main_cc(const std::vector<std::string> &args)
{
	char action = 0;
	std::string action_parameter;
	
	for(std::vector<std::string>::const_iterator i = args.begin(); i != args.end(); ++i)
	{
		if(*i == "-h" || *i == "--help")
		{
			return usage();
		}
		if(*i == "-v" || *i == "--version")
		{
			return version();
		}
		else if(*i == "-c" || *i == "--config")
		{
			if(action) throw fatal_exception("args: Only one action can be specified.");
			action = 'c';			
			++i;
			if(i == args.end()) throw fatal_exception("args: -c misses a filename.");
			action_parameter = *i;
		}
		else if(*i == "-r" || *i == "--restore")
		{
			if(action)
				throw fatal_exception("args: Only one action can be specified.");

			action = 'r';
		}
#ifdef CAP_SCOREBOARD
		else if(*i == "-s" || *i == "--scoreboard")
		{
			if(action)
				throw fatal_exception("args: Only one action can be specified.");
			
			action = 's';
		}
#endif
		else throw fatal_exception("args: Malformed argument \"" + *i + "\".");
	}

	std::auto_ptr<server_interface> state;
	
	switch(action)
	{
		case 'c':
			state = state_from_config(action_parameter);
			break;
		
		case 'r':
			state = state_from_backup();
			break;
			
#ifdef CAP_SCOREBOARD
		case 's':
			// restore the state, but do not open network ports
			state = state_from_backup(false);
			// update the scoreboard
			state->write_scoreboard();
			return 0;
#endif			
		case 0:
			throw fatal_exception("args: -c or -r must be specified, -h for help.");
						
		default:
			throw fatal_exception("args: Internal error.");
	}
	
	if(!state.get())
		throw fatal_exception("setup: Internal error.");
	
	return state->execute();
}

/**
 * \brief The main function.
 *
 * This function calls main_cc with a vector containing the program arguments
 * excluding the first one (the program name). It catches fatal_exception and
 * outputs the exception information to the standard error output.
 *
 * \see main_cc() and fatal_exception
 */
int main(int argc, char *argv[])
{	
	try
	{
		// report errors when std::cout/cerr fail
		std::cout.exceptions(std::cout.exceptions() | std::ios_base::badbit);
		std::cerr.exceptions(std::cerr.exceptions() | std::ios_base::badbit);
				
		// changed to circumvent a fatal bug in MacOS X 10.2 GCC
		// std::vector<std::string> args(argv + 1, argv + argc);
		std::vector<std::string> args(argc - 1);
		std::copy(argv + 1, argv + argc, args.begin());
		return main_cc(args);
	}
	catch(const fatal_exception &e)
	{
		if(!std::cout) std::cout.clear();
		std::cout << e.what() << std::endl;
		return 1;
	}
	catch(const protocol_exception &e)
	{
		if(!std::cout) std::cout.clear();
		std::cout << "main: Protocol exception thrown but not handled:\n"
			<< e.what() << std::endl;
		return 1;
	}
	catch(const std::exception &e)
	{
		if(!std::cout) std::cout.clear();
		std::cout << "main: Unhandled std::exception:\n"
			<< e.what() << std::endl;
		return 1;
	}
	catch(...)
	{
		if(!std::cout) std::cout.clear();
		std::cout << "main: Unhandled unknown exception." << std::endl;
		return 1;
	}
}
