/* $Id: scoreboard.cc,v 1.1 2005/07/11 21:45:39 whess Exp $ */
/* Overlord Server for the Refefree Programming Contest Judging System
 * Copyright (C) 2005 Wolfgang Hess
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
/**
 * \file scoreboard.cc
 * \brief Function implementing the scoreboard.
 *
 * Here the scoreboard function is defined.
 */
#include "caps.hh"

#include <vector>
#include <sstream>
#include <map>
#include <set>
#include <string>
#include <utility>
#include <iostream>
#include <algorithm>

#include "scoreboard.hh"

/**
 * \brief Escaped all XML special characters.
 *
 * This function replaces any occurrences of <, >, &, ' and " with
 * &amp;lt;, &amp;gt;, &amp;amp;, &amp;apos; and &amp;quot; respectively.
 *
 * \param plain the string to work on
 * \return the input string, but this time with the escaped special characters
 */
std::string escape_xml_specials(const std::string &plain)
{
	std::string result;
	
	for(std::string::const_iterator i = plain.begin(); i != plain.end(); ++i)
	{
		switch(*i)
		{
			case '<': result += "&lt;"; break;
			case '&': result += "&amp;"; break;
			case '>': result += "&gt;"; break;
			case '\'': result += "&apos;"; break;
			case '\"': result += "&quot;"; break;
			default:  result += *i;
		}
	}
	
	return result;
}

/**
 * \brief Creates XML output for a scoreboard based on the arguments.
 *
 * If CAP_SCOREBOARD is not defined the function simply returns the
 * empty string. It is not called in this case either.
 *
 * \param c the contest information to be included in the scoreboard
 * \param P the problem set
 * \param T information about all teams
 * \param S information about all submissions made so far
 * \param penalty the penalty for a wrong submission in minutes (usually 20)
 * \return a string containing the (XML) scoreboard
 *
 * \see CAP_SCOREBOARD
 */
std::string create_scoreboard(const scoreboard::contest &c,
	const std::vector<scoreboard::problem> &P,
	const std::vector<scoreboard::team> &T,
	const std::vector<scoreboard::submission> &S,
	int penalty)
{
#ifdef CAP_SCOREBOARD
	std::ostringstream board;
	
	board
		<< "<?xml version=\"1.0\" encoding=\"UTF-8\" ?>\n"
		<< "<?xml-stylesheet href=\"scoreboard.xsl\" type=\"text/xml\" ?>\n"
		<< "<!DOCTYPE scoreboard>\n"
		<< "<scoreboard>\n"
		<< "\t<contest>\n"
		<< "\t\t<name>" << escape_xml_specials(c.description) << "</name>\n"
		<< "\t\t<time>" << c.ctime << "</time>\n"
		<< "\t\t<duration>" << c.duration << "</duration>\n"
		<< "\t</contest>\n"
		<< "\t<standings>\n"
		<< "\t\t<header>\n"
		<< "\t\t\t<csshelper></csshelper>\n"
		<< "\t\t\t<csshelper>Team</csshelper>\n";
		
	for(std::vector<scoreboard::problem>::const_iterator
		i = P.begin(); i != P.end(); ++i)
		board << "\t\t\t<problemid>" << escape_xml_specials(i->id) << "</problemid>\n";
	
	board
		<< "\t\t\t<csshelper>Total</csshelper>\n"
		<< "\t\t</header>\n";
	
	std::map<std::pair<std::string, std::string>, std::pair<int, int> >
		result_map;
	std::set<std::pair<std::string, std::string> >
		blocked_set;
	
	for(std::vector<scoreboard::submission>::const_iterator
		i = S.begin(); i != S.end(); ++i)
	{
		if(blocked_set.count(std::make_pair(i->submitter, i->problem_id)))
			continue;
		
		if(i->state == "accepted" || i->state == "unjudged" || i->state == "new")
		{
			blocked_set.insert(std::make_pair(i->submitter, i->problem_id));
					
			if(i->state == "accepted")
			{
				std::pair<int, int> &j = result_map.insert(std::make_pair(
					std::make_pair(i->submitter, i->problem_id),
					std::make_pair(0, -1))).first->second;
				++j.first;
				j.second = i->stime;
			}
		}
		else if(i->state == "ignored") continue;
		else if(i->state == "rejected")
		{
			// add a wrong submission
			++(result_map.insert(std::make_pair(
				std::make_pair(i->submitter, i->problem_id),
				std::make_pair(0, -1))).first->second.first);
		}
		else
		{
			std::cout << "scoreboard: Malformed judgement " << (i - S.begin()) 
				<< " ignored." << std::endl;
			continue;
		}
	}
	
	std::vector<std::pair<std::pair<int, int>, std::string> > total;
	std::map<std::string, std::string> team_names;
	
	for(std::vector<scoreboard::team>::const_iterator
		i = T.begin(); i != T.end(); ++i)
	{
		std::pair<int, int> t;
		
		for(std::vector<scoreboard::problem>::const_iterator
			j = P.begin(); j != P.end(); ++j)
		{
			std::map<std::pair<std::string, std::string>, std::pair<int, int> >::iterator
				k = result_map.find(std::make_pair(i->id, j->id));
			
			if(k != result_map.end() && k->second.second >= 0)
			{
				--t.first;	// count solved problems negative
				t.second += k->second.second + (k->second.first - 1) * penalty;
			}
		}
		
		total.push_back(std::make_pair(t, i->id));
		team_names.insert(std::make_pair(i->id, i->name));
	}
	
	std::stable_sort(total.begin(), total.end());
	
	int place = 0;
	std::pair<int, int> place_total;
		
	for(std::vector<std::pair<std::pair<int, int>, std::string> >::iterator
		i = total.begin(); i != total.end(); ++i)
	{
		if(i == total.begin() || i->first != place_total)
		{
			place = i - total.begin() + 1;
			place_total = i->first;
		}
		
		board
			<< "\t\t<team>\n"
			<< "\t\t\t<place>" << place << "</place>\n"
			<< "\t\t\t<name>" << escape_xml_specials(team_names.find(i->second)->second)
				<< "</name>\n";
		
		for(std::vector<scoreboard::problem>::const_iterator
			j = P.begin(); j != P.end(); ++j)
		{
			board << "\t\t\t<result problem=\"" << escape_xml_specials(j->id) << "\">";
			
			std::map<std::pair<std::string, std::string>, std::pair<int, int> >::iterator
				k = result_map.find(make_pair(i->second, j->id));
			
			if(k != result_map.end() && k->second.first > 0)
			{
				board << k->second.first << "/";
				
				if(k->second.second == -1)
					board << "-";
				else
					board << k->second.second;
			}
			
			board << "</result>\n";
		}
		
		board
			<< "\t\t\t<total>" << (-(i->first.first)) << "/" << i->first.second << "</total>\n"
			<< "\t\t</team>\n";
	}
			
	board
		<< "\t</standings>\n"
		<< "\t<problems>\n";
			
	for(std::vector<scoreboard::problem>::const_iterator
		i = P.begin(); i != P.end(); ++i)
		board << "\t\t<problem><id>" << escape_xml_specials(i->id)
			<< "</id><name>" << escape_xml_specials(i->name) << "</name></problem>\n";
		
	board
		<< "\t</problems>\n"
		<< "</scoreboard>\n";
	
	return board.str();
#else
	return "";
#endif // CAP_SCOREBOARD
}

