/* $Id: server.cc,v 1.1 2005/07/11 21:45:39 whess Exp $ */
/* Overlord Server for the Refefree Programming Contest Judging System
 * Copyright (C) 2005 Wolfgang Hess
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
/**
 * \file server.cc
 * \brief The implementation of the server.
 *
 * This file contains the complete server implementation along with
 * its helper classes.
 */
#include <string>
#include <iostream>
#include <fstream>
#include <ios>
#include <memory>
#include <list>
#include <map>
#include <set>
#include <vector>
#include <csignal>
#include <sstream>
#include <cassert>
#include <cstdlib>
#include <time.h>
#include <unistd.h>
#include <boost/utility.hpp>
#include <boost/shared_ptr.hpp>

#include "server.hh"
#include "exception.hh"
#include "socket.hh"
#include "caps.hh"
#include "scoreboard.hh"
#include "files.hh"

/**
 * \class server_port
 * \brief This is the class of server port objects used by the server.
 *
 * These objects contain an object of type server_socket_interface and
 * additional information about the port, whether contestants or judges
 * are accepted.
 *
 * Objects of this type are non-copyable.
 */
class server_port : boost::noncopyable
{
	public:
		server_port(boost::uint16_t port_, bool accept_contestants_,
			bool accept_judges_, bool use_local);
		std::auto_ptr<socket_interface> accept() { return server_socket->accept(); }	///< calls server_socket_interface::accept()
		std::string get_flags_string() const;
		bool accepts_contestants() const { return accept_contestants; }	///< returns whether contestant logins are accepted
		bool accepts_judges() const { return accept_judges; }	///< returns whether judge logins are accepted
		
	private:
		std::auto_ptr<server_socket_interface> server_socket;	///< the server socket
		bool accept_contestants;	///< whether contestant logins are accepted
		bool accept_judges;	///< whether judge logins are accepted
};

/**
 * \brief Creates a new server port.
 *
 * This constructor calls create_server_socket() to create the server socket
 * and stores the additional information.
 *
 * \param port_ the port to listen to
 * \param accept_contestants_ whether contestant logins are accepted
 * \param accept_judges_ whether judge logins are accepted
 * \param use_local if true the server socket only listens to 127.0.0.1
 */
server_port::server_port(boost::uint16_t port_, bool accept_contestants_,
	bool accept_judges_, bool use_local)
	: server_socket(create_server_socket(port_, use_local)), 
	accept_contestants(accept_contestants_),
	accept_judges(accept_judges_)
{
}

/**
 * \brief Returns part of the server flags
 *
 * This function returns a string containing the server flags
 * telling the client about the accepted types of the account.
 *
 * \return the contestant and/or judge server flags
 */
std::string server_port::get_flags_string() const
{
	return std::string(accept_contestants ? "contestants " : "")
		+ (accept_judges ? "judges " : "")
		;
}

/**
 * \class account
 * \brief Information about an account.
 *
 * Th information contained in an object of this class is not
 * changed after creation. It contains the password, the real name in
 * case of a contestant and whether the account is a contestant or judge
 * accout. The logins are not stored here but as the keys of the account map.
 *
 * Objects of this type are non-copyable.
 */
class account : boost::noncopyable
{
	public:
		account(const std::string &passwd_, const std::string &name_,
			bool contestant_, bool judge_, bool hide_ = false);
		bool check_passwd(const std::string &input) const;
		std::string get_name() const { return name; }	///< returns the real name of a contestant
		std::string get_flags_string() const;
		bool is_contestant() const { return contestant; }	///< returns whether this is a contestant account
		bool is_judge() const { return judge; }	///< returns whether this is a judge account
		bool is_hidden() const { return hide; } ///< returns whether to hide from the scoreboard
		
	private:
		std::string passwd;	///< password of this account
		std::string name;	///< real name of a contestant
		bool contestant;	///< whether this is a contestant account
		bool judge;	///< whether this is a judge account
		bool hide;	///< whether to hide from the scoreboard
};

/**
 * \brief Create a new account object
 *
 * \param passwd_ password of the account
 * \param name_ real name in case of a contestant account
 * \param contestant_ whether this is a contestant account
 * \param judge_ whether this is a judge account
 * \param hide_ whether to hide this account from the scoreboard
 */
account::account(const std::string &passwd_, const std::string &name_,
	bool contestant_, bool judge_, bool hide_)
	: passwd(passwd_), name(name_), contestant(contestant_), judge(judge_),
	hide(hide_)
{
}

/**
 * \brief Checks against the stored account password.
 *
 * \param input the input to check against
 * \return true if the input is correct
 */
bool account::check_passwd(const std::string &input) const
{
	return input == passwd;
}

/**
 * \brief Returns part of the connection flags
 *
 * This function returns a string containing the connection flags
 * telling the client about the type of the account.
 *
 * \return the contestant or judge flag
 */
std::string account::get_flags_string() const
{
	return std::string(contestant ? "contestant " : "")
		+ (judge ? "judge " : "");
}

/**
 * \class connection
 * \brief An open connection together with corresponding data
 *
 * This class stores information about socket, account, taken locks
 * and up to now unanswered heartbeats.
 */
class connection : boost::noncopyable
{
	public:
		connection(std::auto_ptr<socket_interface> socket_);
		~connection();
		socket_interface &get_socket() { return *socket; }	///< returns the socket of the connection
		void set_account(boost::shared_ptr<account> account_);
		const account &get_account() const;
		
		bool get_submission_lock(int snum);
		bool get_clarification_lock(int cnum);
		bool has_submission_lock(int snum);
		bool has_clarification_lock(int cnum);
		void release_submission_lock(int snum);
		void release_clarification_lock(int cnum);
		int get_locks_number() const { return locks_held.size(); }	///< return number of locks held
		
		void send_heartbeat(int max);
		void received_heartbeat();
		int get_unanswered_heartbeats() const { return unanswered_heartbeats; }	///< return #unanswered_heartbeats
		
		void wait_for_login(int max);
		
	private:
		bool get_lock(int num, const char *prefix);
		bool has_lock(int num, const char *prefix);
		void release_lock(int num, const char *prefix);
			
		std::auto_ptr<socket_interface> socket;	///< the socket of this connection
		boost::shared_ptr<account> acct;	///< the account for which this is a connection, or null if not logged in
		std::set<std::pair<int, std::string> > locks_held;	///< all locks held by this connection

		int unanswered_heartbeats;	///< number of up to now unanswered heartbeats			
		int skipped_login_heartbeats;	///< number of heartbeats skipped before login
};

/**
 * \brief Create a newly accepted connection with a socket
 *
 * \param socket_ the socket of the connection
 */
connection::connection(std::auto_ptr<socket_interface> socket_)
	: socket(socket_), acct(), locks_held(), unanswered_heartbeats(0),
	skipped_login_heartbeats(0)
{
	if(!socket.get()) throw fatal_exception("internal: Connection created without socket.");
}

/**
 * \brief Destructor
 *
 * Locks that are still taken when the connection is destroyed are automatically
 * released.
 */
connection::~connection()
{
	while(!locks_held.empty())
	{
		std::string prefix = locks_held.begin()->second;
		if(prefix == "") release_submission_lock(locks_held.begin()->first);
		else if(prefix == "C") release_clarification_lock(locks_held.begin()->first);
		else std::cout << "connection: lock has unknown lock type." << std::endl;
	}
}
/**
 * \brief Sets the account
 *
 * This function changes the account of this connection.
 *
 * \param account_ the account used by this connection
 */
void connection::set_account(boost::shared_ptr<account> account_)
{
	acct = account_;
}

/**
 * \brief Returns the account used by the connection.
 *
 * This function should only be called after the account has been set.
 *
 * \return the acoount used by this connection
 */
const account &connection::get_account() const
{
	assert(acct.get() != 0);
	return *acct;
}

/**
 * \brief Tries to take a submission lock.
 *
 * \param snum the number of the submission for which the lock should be taken
 * \return whether the lock is taken successfully or not
 *
 * \see get_lock()
 */
bool connection::get_submission_lock(int snum)
{
	return get_lock(snum, "");
}

/**
 * \brief Tries to take a clarification lock.
 *
 * \param cnum the number of the clarification request for which the lock should be taken
 * \return whether the lock is taken successfully or not
 *
 * \see get_lock()
 */
bool connection::get_clarification_lock(int cnum)
{
	return get_lock(cnum, "C");
}

/**
 * \brief Checks whether this connection has a specific submission lock
 *
 * \param snum the number of the submission for which the lock should be checked
 * \return whether the lock is taken by this connection
 *
 * \see has_lock()
 */
bool connection::has_submission_lock(int snum)
{
	return has_lock(snum, "");
}

/**
 * \brief Checks whether this connection has a specific clarification lock
 *
 * \param cnum the number of the clarification request for which the lock
 *     should be checked
 * \return whether the lock is taken by this connection
 *
 * \see has_lock()
 */
bool connection::has_clarification_lock(int cnum)
{
	return has_lock(cnum, "C");
}

/**
 * \brief Releases a lock to a submission that this connection has taken before.
 *
 * This function should only be called if this connection has taken the lock
 * before and not yet released it.
 *
 * \param snum the number of the submission whose lock to release
 *
 * \see release_lock()
 */
void connection::release_submission_lock(int snum)
{
	release_lock(snum, "");
}

/**
 * \brief Releases a lock to a clarification that this connection has taken before.
 *
 * This function should only be called if this connection has taken the lock
 * before and not yet released it.
 *
 * \param cnum the number of the clarification request whose lock to release
 *
 * \see release_lock()
 */
void connection::release_clarification_lock(int cnum)
{
	release_lock(cnum, "C");
}

/**
 * \brief Take a lockfile based on number and prefix
 *
 * This function has a common interface for submission and clarification
 * locks.
 *
 * \param num number of the submission or clarification whose lock to take
 * \param prefix either empty for submission locks or 'C' for clarifications.
 * \return whether the lock is taken successfully or not
 */
bool connection::get_lock(int num, const char *prefix)
{
	std::ostringstream lock_fs;
	lock_fs << prefix << num << ".lock";
	const std::string lock_str(lock_fs.str());
	const char * const lock_file = lock_str.c_str();

	if(get_lock_file(lock_file))	// lock taken?
	{
		locks_held.insert(std::make_pair(num, std::string(prefix)));		
		return true;
	}
	else return false;	// we did not get the lock
}

/**
 * \brief Checks whether a lockfile is currently taken by this connection
 *
 * This function has a common interface for submission and clarification
 * locks.
 *
 * \param num number of the submission or clarification whose lock to check
 * \param prefix either empty for submission locks or 'C' for clarifications.
 * \return whether the lock is currently taken by this connection
 */
bool connection::has_lock(int num, const char *prefix)
{
	return locks_held.count(std::make_pair(num, std::string(prefix))) != 0;
}

/**
 * \brief Releases a lockfile this connection has taken before
 *
 * This function has a common interface for submission and clarification
 * locks.
 *
 * \param num number of the submission or clarification whose lock to release
 * \param prefix either empty for submission locks or 'C' for clarifications.
 */
void connection::release_lock(int num, const char *prefix)
{
	if(locks_held.count(std::make_pair(num, std::string(prefix))) == 0)
		throw fatal_exception("connection: lock released but not held before.");

	std::ostringstream lock_fs;
	lock_fs << prefix << num << ".lock";
	const std::string lock_str(lock_fs.str());
	const char * const lock_file = lock_str.c_str();
	
	release_lock_file(lock_file);
	
	locks_held.erase(std::make_pair(num, std::string(prefix)));
}

/**
 * \brief Sends a heartbeat request.
 *
 * If the maximal number of unanswered heartbeat requests is not yet reached,
 * this function sends a heartbeat request and increments the number of
 * unanswered requests. Otherwise a protocol_exception is thrown.
 *
 * \param max the maximal number of unanswered heartbeat requests
 * 
 * \see unanswered_heartbeats, protocol_exception
 */
void connection::send_heartbeat(int max)
{
	if(unanswered_heartbeats >= max)
		throw protocol_exception("Connection timed out");
	socket->send("heartbeat_request\n");
	++unanswered_heartbeats;
}

/**
 * \brief Account for a received heartbeat reply.
 *
 * If there are unanswered heartbeat requests, their number is decremented.
 * Otherwise a protocol_exception is thrown.
 *
 * \see unanswered_heartbeats, protocol_exception
 */
void connection::received_heartbeat()
{
	if(unanswered_heartbeats == 0)
		throw protocol_exception("Unexpected heartbeat reply");
	--unanswered_heartbeats;
}

/**
 * \brief Counts heartbeats skipped because of login.
 *
 * During login no heartbeats are sent. Therefore if a heartbeat is sent
 * to all clients that are already logged in the #skipped_login_heartbeats
 * of all clients that are not is incremented. If it reaches the maximal
 * number of unanswered heartbeats a protocol_exception is thrown. This
 * implements the timeout before login.
 *
 * \param max the maximal number of unanswered heartbeat requests
 * 
 * \see skipped_login_heartbeats, protocol_exception
 */
void connection::wait_for_login(int max)
{
	if(skipped_login_heartbeats >= max)
		throw protocol_exception("Login timed out");
	++skipped_login_heartbeats;
}

/**
 * \class internal_state
 * \brief The main server class.
 *
 * This class implements the complete server functionality. The complete
 * internal state is contained herein.
 *
 * Internal timing is done by #last_second. Some actions are done every second
 * and the second they were done last is stored in #last_second, so that by
 * comparison we can find out whether we are in a new second by now.
 *
 * The contest timing is done by #seconds_into_this_minute to be sure not to
 * miss any elapsed seconds. This is the time since the Epoch modulo 60, which
 * is the seconds in this minute. The elapsed contest seconds are stored
 * in #elapsed_seconds. This together ensures that the server has to hang for
 * more than a minute to forget counting some seconds and also that whole day
 * jumps in time do not effect the outcome (since seconds since the Epoch do
 * not account for some of the leap years, see POSIX). We do miss leap seconds.
 * #seconds_into_this_minute is only valid when the contest is running, i.e.
 * when #contest_state is cs_running.
 *
 * The signal handling part makes it impossible to have more than one object at
 * the same time. It could be moved to execute() though. signal_hup() is unsafe
 * when the signal handling is executed in another thread as execute().
 */
class internal_state : public server_interface
{
	public:
		internal_state();
		~internal_state();
		int execute();
		
		void set_configfile(const std::string &configfile_);
		void set_description(const std::string &description_);
		void set_duration(int duration_);
		void set_scoreboard_duration(int scoreboard_duration_);
		void set_penalty(int penalty_);
		void set_max_message_size(int max_message_size_);
		void set_heartbeat_interval(int heartbeat_interval_);
		void set_connection_timeout(int connection_timeout_);
		void set_auto_finish(bool auto_finish_);
		void add_port(boost::uint16_t port, bool accept_contestants,
			bool accept_judges, bool use_local);
		void add_contestant(const std::string &login, 
			const std::string &passwd, const std::string &name,
			bool hide);
		void add_judge(const std::string &login,
			const std::string &passwd);
		void add_language(const std::string &id,
			const std::string &name);
		void add_problem(const std::string &id,
			const std::string &name);
		
		void restore_backup(const std::string &contest_state_,
			int elapsed_seconds_);
		void write_scoreboard();
		
	private:
		typedef std::list<boost::shared_ptr<server_port> > port_list;	///< type of #ports
		typedef std::map<std::string, boost::shared_ptr<account> > account_map;	///< type of #accounts
		typedef std::list<std::pair<boost::shared_ptr<connection>, boost::shared_ptr<server_port> > > new_connection_list;	///< type of #new_connections
		typedef std::map<std::string, boost::shared_ptr<connection> > connection_map;	///< type of #connections
		typedef std::map<std::string, std::string> language_map;	///< type of #languages
		typedef std::map<std::string, std::string> problem_map;	///< type of #problems

		std::string configfile;	///< name of the configuration file, see set_configfile() 
		std::string description;	///< name of the contest, see set_description()
		int duration;	///< duration of the contest, see set_duration()
		int scoreboard_duration;	///< duration of the scoreboard, see set_scoreboard_duration()
		int penalty;	///< penalty for wrong submissions, see set_penalty()
		int max_message_size;	///< maximal accepted message size, see set_max_message_size()
		int heartbeat_interval;	///< heartbeat interval, see set_heartbeat_interval()
		int connection_timeout;	///< connection time-out, see set_connection_timeout()
		bool auto_finish;	///< automatically set the state to 'after' at the end
		port_list ports;	///< a list containing all #server_port objects
		account_map accounts;	///< a list containing all #account objects
		new_connection_list new_connections;	///< a list of all connections not yet logged in (without an account)
		connection_map connections;	///< a list of all connections that are logged in (with an account)
		language_map languages;	///< a list of language id and name pairs
		problem_map problems;	///< a list of problem id and name pairs
		
	private:
		int next_submission_number;	///< current number of submissions
#ifdef CAP_CLARIFICATIONS
		int next_clarification_number;	///< current number of clarification requests
#endif
		bool terminate;	///< did we get a signal to terminate?
		/**
		 * \enum cs_t
		 * \brief all different contest states
		 */
		enum cs_t { cs_before = 0, cs_running, cs_paused, cs_after };
		cs_t contest_state;	///< current contest state
		int elapsed_seconds;	///< seconds elapsed... (in running contest state)
		int seconds_into_this_minute;	///< ...at this moment
		time_t last_second;	///< time when the last second was processed
		int seconds_before_heartbeat;	///< number of seconds until the next heartbeat is sent
		std::string cached_scoreboard;	///< the current scoreboard
		
		int get_time();	// time in seconds since contest start
		std::string create_server_state_backup();
		void autosave_server_state_backup();
		std::string create_scoreboard_string() const;
		void autoupdate_scoreboard();
		std::string create_serverstatus_string() const;
		void notify_contestants();
		void send_submission_results(const std::string &login,
			socket_interface &socket);
		void send_submission_list(socket_interface &socket);
		void notify_judges();

		void send_clarification_replies(const std::string &login,
			socket_interface &socket);
		void send_clarification_list(socket_interface &socket);		
		void notify_clarifications_questioner();
		void notify_clarifications_others();

	private:
		static internal_state *that;	///< pointer used by the static signal handlers
		typedef void (*sighandler_t)(int);      ///< define our own sighandler_t for compatibility
		static sighandler_t 
			old_int,	///< old signal handler changed by the server
			old_term,	///< old signal handler changed by the server
			old_hup;	///< old signal handler changed by the server
		static void static_signal_int(int);	///< signal handler, calls signal_int()
		static void static_signal_hup(int);	///< signal handler, calls signal_hup()
		void signal_int();	///< called by static_signal_int(), interrupting shuts down the server,
		void signal_hup();	///< called by static_signal_hup(), hang up starts/pauses/continues contest
		void change_state(cs_t newstate);	///< called to change the contest_state
};
	
internal_state::internal_state()
	: configfile(), description(), duration(0), scoreboard_duration(0),
	penalty(0), max_message_size(0), heartbeat_interval(1),
	connection_timeout(0), auto_finish(false),
	next_submission_number(0),
#ifdef CAP_CLARIFICATIONS
	next_clarification_number(0),
#endif
	terminate(false), contest_state(cs_before),
	elapsed_seconds(0), seconds_into_this_minute(0),
	last_second(time_t(-1)), seconds_before_heartbeat(1),
	cached_scoreboard()
{
	// we succeeded in constructing this server and now bind some signals
	if(that != 0) // the server should be unique
		throw fatal_exception("internal: Server creation called twice.");
	
	that = this;
	
	// the server stops on ctrl+c and changes state on hang-up signals
	old_int = signal(SIGINT, static_signal_int);
	old_term = signal(SIGTERM, static_signal_int);
	old_hup = signal(SIGHUP, static_signal_hup);
	
}

internal_state::~internal_state()
{
	new_connections.clear();
	connections.clear();
	
	// std::cout << "Waiting a heartbeat interval (" << heartbeat_interval << " second"
	// 	<< (heartbeat_interval != 1 ? "s" : "") << ") for closing "
	// 	<< "network connections." << std::endl;
	// 
	// seconds_before_heartbeat = heartbeat_interval; 
	// for(;;)
	// {
	// 	time_t current_time = time(NULL);
	// 	if(current_time != last_second)
	// 	{
	// 		last_second = current_time;
	// 		if(seconds_before_heartbeat == 0) break;
	// 		--seconds_before_heartbeat;
	// 	}
	// }
	
	// restore signals
	signal(SIGINT, old_int);
	signal(SIGTERM, old_term);
	signal(SIGHUP, old_hup);
	
	that = 0;
}

namespace
{
	/**
	 * \brief Remove CR characters (0x0c) from the first lines of a string.
	 *
	 * If CAP_CR_TOLERANT is defined, this function removes all
	 * CR characters (value 0x0c) from a certain number of lines
	 * from the beginning of the input, otherwise it has no effect
	 *
	 * \param input string from which the CR should be removed
	 * \param n number of lines from which to remove the characters
	 *
	 * \see CAP_CR_TOLERANT
	 */
	void strip_cr(std::string &input, size_t n)
	{
	#ifdef CAP_CR_TOLERANT
		// we want to strip all CR (0x0c) in the first n lines (separated by 0x0a)
		std::string output;
		std::string::iterator i = input.begin();
		for(size_t j = 0; j != n && i != input.end(); ++i)
		{
			if(*i != '\r') output += *i; 	// strip CR
			if(*i == '\n') ++j;			// advance to next line j
		}

		// copy the rest of the input unaltered
		output.insert(output.end(), i, input.end());

		// apply changes
		std::swap(input, output);
	#endif	
	}

	/**
	 * \brief Remove CR characters (0x0c) from a string.
	 *
	 * If CAP_CR_TOLERANT is defined, this function removes all
	 * CR characters (value 0x0c) from the input, otherwise it
	 * has no effect.
	 *
	 * \param input string from which the CR should be removed
	 *
	 * \see CAP_CR_TOLERANT
	 */
	void strip_cr(std::string &input)
	{
	#ifdef CAP_CR_TOLERANT
		// we want to strip all CR (0x0c)
		std::string output;
		for(std::string::iterator i = input.begin(); i != input.end(); ++i)
			if(*i != '\r') output += *i; 	// strip CR

		// apply changes
		std::swap(input, output);
	#endif	
	}
}

int internal_state::execute()
{
	while(!terminate)	// break on signal
	{
		bool new_second = false;
		bool send_heartbeats = false;
		
		// check whether we're in a new second
		{
			time_t current_time = time(NULL);
			if(current_time != last_second) 
			{
				last_second = current_time;
				new_second = true;
			}
			
			if(new_second)
			{
				--seconds_before_heartbeat;
				if(seconds_before_heartbeat == 0)
				{
					send_heartbeats = true;
					seconds_before_heartbeat = heartbeat_interval;
				}
			}
		}
		
		// accept new connections
		for(port_list::const_iterator i = ports.begin(); i != ports.end(); ++i)
		{
			std::auto_ptr<socket_interface> new_client;
			while((new_client = (*i)->accept()).get())
			{
				// send server prologue
				new_client->send(std::string(
					"hello\n")
					+ CAP_VERSION + "\n"
					+ description + "\n"
					+ (*i)->get_flags_string() + "\n"
					);
				
				boost::shared_ptr<connection> new_connection(
					new connection(new_client));
				new_connections.push_back(std::make_pair(new_connection, *i));
			}
		}
		
		// process new connections
		for(new_connection_list::iterator i = new_connections.begin();
			i != new_connections.end();)
		{
			socket_interface &socket = i->first->get_socket();
			try
			{
#ifdef CAP_CONNECTION_TIMEOUT
				if(connection_timeout > 0 && send_heartbeats)
					i->first->wait_for_login(connection_timeout);
#endif
				
				if(!socket.process())	// check for eof
				{
					new_connections.erase(i++);
				}
#ifdef CAP_MAX_MESSAGE_SIZE
				else if(socket.message_too_large(max_message_size))
				{
					throw protocol_exception("Maximal message size exceeded");
				}
#endif
				else
				{
					std::string in;
					if(socket.receive(in))
					{
						strip_cr(in);
						
						std::istringstream is(in);
						std::string cmd, flags, login, passwd;
						if(!(getline(is, cmd) && cmd == "login_request"
							&& getline(is, flags)
							&& getline(is, login) && getline(is, passwd)))
							throw protocol_exception("Malformed login");
						
						std::set<std::string> flagsmap;
						{
							std::istringstream is(flags);
							std::string flag;
							while(is >> std::ws >> flag) flagsmap.insert(flag);
						}
						const bool contestant_client = (flagsmap.erase("contestant") != 0);
						const bool judge_client = (flagsmap.erase("judge") != 0);
						if(!flagsmap.empty())
							throw protocol_exception("Malformed flags");
						
						// search account
						account_map::iterator j = accounts.find(login);
						if(j == accounts.end())
							throw protocol_exception("Login name does not exist");
						
						if(j->second->is_contestant() && !contestant_client)
							throw protocol_exception("Login as contestant is not supported by this client");						
						if(j->second->is_judge() && !judge_client)
							throw protocol_exception("Login as judge is not supported by this client");
						
						// check port permission
						if(!((j->second->is_contestant() && i->second->accepts_contestants())
							|| (j->second->is_judge() && i->second->accepts_judges())))
							throw protocol_exception("Login on wrong port");
						
						// we do not allow contestant logins after the contest is over
						if(j->second->is_contestant() && contest_state == cs_after)
							throw protocol_exception("Contestant login after contest");
						
						// check the password
						if(!j->second->check_passwd(passwd))
							throw protocol_exception("Password authentication failed");
						
						// check if the account is free
						if(connections.count(login))
							throw protocol_exception("Login '" + login + "' already in use");
						
						// client is accepted
						i->first->set_account(j->second);
						connections.insert(make_pair(login, i->first));
						new_connections.erase(i++);	// connection has been handled correctly

#ifdef CAP_LOGLOGINS
						std::cout << "login: '" << login << "' logged in successfully from "
							<< socket.get_debug_string() << "." << std::endl;
#endif
						
						socket.send(std::string(
							"login_welcome\n")
							+ j->second->get_name() + "\n"
							+ j->second->get_flags_string()
#ifdef CAP_NOTIFIES
								+ "notifies "
#endif
#ifdef CAP_STATUS
								+ "status "
#endif
#ifdef CAP_AUTOCONFIG
								+ "autoconfig "
#endif
#ifdef CAP_SCOREBOARD
								+ "scoreboard "
#endif
#ifdef CAP_CLARIFICATIONS
								+ "clarifications "
#endif
#ifdef CAP_PAUSE
								+ (j->second->is_judge() ? "pause " : "")
#endif
#ifdef CAP_SERVERSTATUS
								+ (j->second->is_judge() ? "serverstatus " : "")
#endif 
								+ "\n"
							);
												
#ifdef CAP_AUTOCONFIG
						for(language_map::iterator i = languages.begin(); i != languages.end(); ++i)
							socket.send(std::string(
								"login_autoconfig\n")
								+ "language\n"
								+ i->first + "\n"
								+ i->second + "\n"
								);
						
						for(problem_map::iterator i = problems.begin(); i != problems.end(); ++i)
							socket.send(std::string(
								"login_autoconfig\n")
								+ "problem\n"
								+ i->first + "\n"
								+ i->second + "\n"
								);
						
						socket.send("login_autoconfig\ncompleted\n");
#endif
						
#ifdef CAP_NOTIFIES
						if(j->second->is_contestant())
							send_submission_results(login, socket);
						if(j->second->is_judge())
							send_submission_list(socket);
#ifdef CAP_CLARIFICATIONS
						send_clarification_replies(login, socket);
						if(j->second->is_judge())
							send_clarification_list(socket);
#endif				
#endif
					}
					else ++i;	// continue since neither eof nor data received					
				}
			}
			catch(const protocol_exception &e)
			{
				// write an error string to the output buffer
				socket.send(std::string("error\n") + e.what() + ".\n");
				socket.process();	// try to send the error message
				
				// connection has been handled correctly
				std::cout << "login[" << socket.get_debug_string() << "]: " << e.what() << "." << std::endl;
				new_connections.erase(i++);	
			}
		}

		// process existing connections		
		for(connection_map::iterator i = connections.begin();
			i != connections.end();)
		{
			socket_interface &socket = i->second->get_socket();
			const account &acct = i->second->get_account();
			const std::string &login = i->first;
						
			try
			{
#ifdef CAP_CONNECTION_TIMEOUT
				if(connection_timeout > 0 && send_heartbeats)
					i->second->send_heartbeat(connection_timeout);
#endif

				if(!socket.process())	// check for eof
				{
					connections.erase(i++);
				}
				else
				{
					std::string in;
#ifdef CAP_MAX_MESSAGE_SIZE
					while(socket.message_too_large(max_message_size) ?
						throw protocol_exception("Maximal message size exceeded") :
						socket.receive(in))
#else
					while(socket.receive(in))
#endif
					{
						const std::string in0 = in;
						strip_cr(in);
						std::istringstream is(in);
						std::string cmd;
						if(!getline(is, cmd))
							throw protocol_exception("Message without protocol code");
						
						if(cmd == "heartbeat_request")
						{
#ifdef CAP_STATUS
							// answer with the current status							
							std::ostringstream answer;
							answer << "heartbeat_whoomp\n";
							
							int ctime = get_time() / 60;
							
							switch(contest_state)
							{
								case cs_before: answer << "before\n"; break;
								case cs_running: answer << "running\n"; break;
								case cs_paused: answer << "paused\n"; break;
								case cs_after: answer << "after\n"; break;
							}
							
							answer << ctime << "\n"
								<< duration << "\n";
							socket.send(answer.str());
#else
							// answer a heartbeat request
							socket.send("heartbeat_whoomp\n");
#endif
						}
#ifdef CAP_CONNECTION_TIMEOUT
						else if(cmd == "heartbeat_whoomp")
						{
							i->second->received_heartbeat();
						}
#endif
						else if(cmd == "submission_submit")
						{
							if(!acct.is_contestant())
								throw protocol_exception("Submission by a non-contestant");
							
							// use a custom 3-line-CR-stripped version of the input here
							std::string in2 = in0;
							strip_cr(in2, 3);
							std::istringstream is2(in2);
							std::string cmd;
							getline(is2, cmd);
							
							if(contest_state == cs_before)
								throw protocol_exception("Submission before the contest");
							if(contest_state == cs_after)
								throw protocol_exception("Submission after the contest");
							
							std::string problem_id;
							if(!getline(is2, problem_id))
								throw protocol_exception("Submission without problem id");
							if(problems.count(problem_id) == 0)
								throw protocol_exception("Submission with unknown problem id");
							
							std::string language_id;
							if(!getline(is2, language_id))
								throw protocol_exception("Submission without language id");
							if(languages.count(language_id) == 0)
								throw protocol_exception("Submission with unknown language id");
							
							std::string source;
							{
								char ch = 0;
								while(is2.get(ch))
									source += ch;
							}
							
							handle_submission(next_submission_number, login,
								problem_id, language_id, source, get_time() / 60);
							// increment the submission number
							++next_submission_number;
						}
						else if(cmd == "submission_results")
						{
							if(!acct.is_contestant())
								throw protocol_exception("Submission results queried by a non-contestants");
							
							send_submission_results(login, socket);
						}
#ifdef CAP_SCOREBOARD
						else if(cmd == "scoreboard_request")
						{
							if(acct.is_judge())
								socket.send("scoreboard\n" + create_scoreboard_string());
							else
								socket.send("scoreboard\n" + cached_scoreboard);
						}
#endif
						else if(cmd == "submission_list")
						{
							if(!acct.is_judge())
								throw protocol_exception("Submission list queried by a non-judge");
							
							send_submission_list(socket);
						}
						else if(cmd == "submission_fetch")
						{
							if(!acct.is_judge())
								throw protocol_exception("Submission fetched by a non-judge");
							
							int snum;
							{
								std::string snum_str;
								if(!getline(is, snum_str))
									throw protocol_exception("Submission fetched without submission id");
								std::istringstream snum_is(snum_str);
								if(!(snum_is >> snum) || snum < 0
									|| snum >= next_submission_number)
									throw protocol_exception("Submission fetched with illegal submission id");
							}
							
							std::ostringstream answer;
							answer << "submission_source\n"
								<< snum << "\n";
							
							if(i->second->get_submission_lock(snum))
							{
								std::string source;
								get_submission_source(snum, source);
								answer << "success\n"
									<< source;
							}
							else
							{
								answer << "failure\n";
							}
							socket.send(answer.str());
						}
						else if(cmd == "submission_judge")
						{
							if(!acct.is_judge())
								throw protocol_exception("Submission judged by a non-judge");
							
							int snum;
							{
								std::string snum_str;
								if(!getline(is, snum_str))
									throw protocol_exception("Submission judged without submission id");
								std::istringstream snum_is(snum_str);
								if(!(snum_is >> snum) || snum < 0
									|| snum >= next_submission_number)
									throw protocol_exception("Submission judged with illegal submission id");
							}
							
							if(!i->second->has_submission_lock(snum))
								throw protocol_exception("Submission judged but no lock held");
							
							std::string verdict;
							if(!getline(is, verdict))
								throw protocol_exception("Submission judged without verdict");
							if(verdict != "new" && verdict != "unjudged" && verdict != "rejected"
								&& verdict != "accepted" && verdict != "ignored" && verdict != "")
								throw protocol_exception("Submission judged with illegal verdict");
							
							if(verdict != "")
							{
								std::string reason;
								if(!getline(is, reason))
									throw protocol_exception("Submission judged without reason");
							
								new_submission_state(snum, verdict, reason, login);
							}
							
							i->second->release_submission_lock(snum);
						}
#ifdef CAP_PAUSE
						else if(cmd == "pause_toggle")
						{
							if(!acct.is_judge())
								throw protocol_exception("State change ordered by a non-judge");
							
							std::string newstate;
							if(!getline(is, newstate))
								throw protocol_exception("State change ordered without a new state");
							
							if(newstate == "before") change_state(cs_before);	
							else if(newstate == "running") change_state(cs_running);	
							else if(newstate == "paused") change_state(cs_paused);	
							else if(newstate == "after") change_state(cs_after);
							else throw protocol_exception("State change ordered to unknown state");
						}
#endif
#ifdef CAP_CLARIFICATIONS
						else if(cmd == "clarification_request")
						{
							// use a custom 2-line-CR-stripped version of the input here
							std::string in2 = in0;
							strip_cr(in2, 2);
							std::istringstream is2(in2);
							std::string cmd;
							getline(is2, cmd);
							
							if(contest_state == cs_after)
								throw protocol_exception("Clarification after the contest");
							
							std::string problem_id;
							if(!getline(is2, problem_id))
								throw protocol_exception("Clarification without problem id");
							if(!problem_id.empty() && problems.count(problem_id) == 0)
								throw protocol_exception("Clarification with unknown problem id");
							
							std::string question;
							{
								char ch = 0;
								while(is2.get(ch))
									question += ch;
							}
							
							handle_clarification(next_clarification_number, login,
								problem_id, question, get_time() / 60);
							// increment the clarification number
							++next_clarification_number;
						}
						else if(cmd == "clarification_replies")
						{
							send_clarification_replies(login, socket);
						}
						else if(cmd == "clarification_list")
						{
							if(!acct.is_judge())
								throw protocol_exception("Clarification list queried by a non-judge");
							
							send_clarification_list(socket);
						}
						else if(cmd == "clarification_lock")
						{
							if(!acct.is_judge())
								throw protocol_exception("Clarification locked by a non-judge");
							
							int cnum;
							{
								std::string cnum_str;
								if(!getline(is, cnum_str))
									throw protocol_exception("Clarification locked without request id");
								std::istringstream cnum_is(cnum_str);
								if(!(cnum_is >> cnum) || cnum < 0
									|| cnum >= next_clarification_number)
									throw protocol_exception("Clarification locked with illegal request id");
							}
							
							std::ostringstream answer;
							answer << "clarification_locked\n"
								<< cnum << "\n";
							
							if(i->second->get_clarification_lock(cnum))
								answer << "success\n";
							else
								answer << "failure\n";
							
							socket.send(answer.str());
						}
						else if(cmd == "clarification_answer")
						{
							// use a custom 3-line-CR-stripped version of the input here
							std::string in2 = in0;
							strip_cr(in2, 3);
							std::istringstream is2(in2);
							std::string cmd;
							getline(is2, cmd);
							
							if(!acct.is_judge())
								throw protocol_exception("Clarification answered by a non-judge");
							
							int cnum;
							{
								std::string cnum_str;
								if(!getline(is2, cnum_str))
									throw protocol_exception("Clarification answered without request id");
								std::istringstream cnum_is(cnum_str);
								if(!(cnum_is >> cnum) || cnum < 0
									|| cnum >= next_clarification_number)
									throw protocol_exception("Clarification answered with illegal request id");
							}
							
							if(!i->second->has_clarification_lock(cnum))
								throw protocol_exception("Clarification answered but no lock held");
							
							std::string state;
							if(!getline(is2, state))
								throw protocol_exception("Clarification answered without state");
							if(state != "unanswered" && state != "questioner" && state != "everyone" && state != "")
								throw protocol_exception("Clarification answered with illegal state");
							
							if(state != "")
							{
								std::string answer;
								{
									char ch = 0;
									while(is2.get(ch))
										answer += ch;
								}

								new_clarification_state(cnum, state, get_time() / 60, login, answer);
							}
							
							i->second->release_clarification_lock(cnum);
						}
#endif	// CAP_CLARIFICATIONS
#ifdef CAP_SERVERSTATUS
						else if(cmd == "serverstatus_request")
						{
							if(!acct.is_judge())
								throw protocol_exception("Server status request by a non-judge");
							
							std::string reply(create_serverstatus_string());
							socket.send("serverstatus_reply\n" + reply);
						}
#endif
						else	// unknown codes are simply ignored						
						{
							std::cout << "connection[" << i->first <<
								"]: unknown protocol code ignored." << std::endl;
						}
					}

					++i;	// connection handled successfully
				}
			}	
			catch(const protocol_exception &e)
			{
				// write an error string to the output buffer
				socket.send(std::string("error\n") + e.what() + ".\n");
				socket.process();	// try to send the error message
				
				// malformed connection
				std::cout << "connection[" << i->first << "@"
					<< socket.get_debug_string() << "]: "
					<< e.what() << "." << std::endl;
				connections.erase(i++);	
			}									
		}
		
		usleep(10000);	// TODO: select on a sensible set of signals/sockets
		get_time();	// call the function, so that time triggered events work
		
		if(new_second)
		{
			// others are notified first
			notify_clarifications_others();
			notify_clarifications_questioner();
			
			// notify judges first
			notify_judges();
			
			if(contest_state == cs_running || contest_state == cs_paused)	// notify contestants where possible
				notify_contestants();
		}
	}
		
	if(save_server_state_backup(create_server_state_backup()))
		std::cout << "State successfully saved." << std::endl;
	else
	{
		std::cout << "Failed saving the state. Printing data intended for the backup.overlord file\n"
			<< "to the standard error output and terminating:\n"
			<< create_server_state_backup()
			<< std::flush;
	}
	
	return 0;
}

void internal_state::set_configfile(const std::string &configfile_)
{
	configfile = configfile_;
}

void internal_state::set_description(const std::string &description_)
{
	description = description_;
	std::cout << "Contest '" << description << "'." << std::endl;
}

void internal_state::set_duration(int duration_)
{
	duration = duration_;
	std::cout << "Duration is " << duration << " minutes." << std::endl;
}

void internal_state::set_scoreboard_duration(int scoreboard_duration_)
{
	scoreboard_duration = scoreboard_duration_;
#ifdef CAP_SCOREBOARD
	std::cout << "Scoreboard duration is " << scoreboard_duration << " minutes." << std::endl;
#endif
}

void internal_state::set_penalty(int penalty_)
{
	penalty = penalty_;
#ifdef CAP_SCOREBOARD
	std::cout << "Penalty is " << penalty << " minutes." << std::endl;
#endif
}

void internal_state::set_max_message_size(int max_message_size_)
{
	max_message_size = max_message_size_;
#ifdef CAP_MAX_MESSAGE_SIZE
	std::cout << "Maximal message size is " << max_message_size << " kilobytes." << std::endl;
#endif
}

void internal_state::set_heartbeat_interval(int heartbeat_interval_)
{
	heartbeat_interval = heartbeat_interval_;
#ifdef CAP_CONNECTION_TIMEOUT
	std::cout << "Heartbeat interval is " << heartbeat_interval << " seconds." << std::endl;
#endif
}

void internal_state::set_connection_timeout(int connection_timeout_)
{
	connection_timeout = connection_timeout_;
#ifdef CAP_CONNECTION_TIMEOUT
	if(connection_timeout > 0)
		std::cout << "Connection timeout is " << connection_timeout << " heartbeats." << std::endl;
	else
		std::cout << "Connection timeout is disabled." << std::endl;
#endif
}

void internal_state::set_auto_finish(bool auto_finish_)
{
	auto_finish = auto_finish_;
	std::cout << "Auto-finish is " << (auto_finish ? "on" : "off") << "." << std::endl;
}

void internal_state::add_port(boost::uint16_t port, bool accept_contestants,
	bool accept_judges, bool use_local)
{
	std::cout << "Listening to port " << port << (use_local ? " (local)." : ".") << std::endl;
	
	boost::shared_ptr<server_port> new_port(
		new server_port(port, accept_contestants, accept_judges, use_local));
	
	ports.push_back(new_port);
}

void internal_state::add_contestant(const std::string &login, 
	const std::string &passwd, const std::string &name, bool hide)
{
	std::cout << "Adding contestant '" << login << "' aka '" << name << "'"
		<< (hide ? " (hidden from scoreboard)" : "") << "." << std::endl;
	boost::shared_ptr<account> new_account(new account(passwd, name, true, false, hide));
	if(!accounts.insert(make_pair(login, new_account)).second)
		throw fatal_exception("server: Duplicate of this login name exists.");
}

void internal_state::add_judge(const std::string &login,
	const std::string &passwd)
{
	std::cout << "Adding judge '" << login << "'." << std::endl;
	boost::shared_ptr<account> new_account(new account(passwd, login, false, true));
	if(!accounts.insert(make_pair(login, new_account)).second)
		throw fatal_exception("server: Duplicate of this login name exists.");
}

void internal_state::add_language(const std::string &id,
	const std::string &name)
{
	std::cout << "Adding language '" << id << "' aka '" << name << "'." << std::endl;
	if(!languages.insert(make_pair(id, name)).second)
		throw fatal_exception("server: Duplicate of this language id exists.");
}

void internal_state::add_problem(const std::string &id,
	const std::string &name)
{
	std::cout << "Adding problem '" << id << "' aka '" << name << "'." << std::endl;
	if(!problems.insert(make_pair(id, name)).second)
		throw fatal_exception("server: Duplicate of this problem id exists.");
}

void internal_state::restore_backup(const std::string &contest_state_,
	int elapsed_seconds_)
{
	if(contest_state_ == "before") contest_state = cs_before;
	else if(contest_state_ == "running") contest_state = cs_running;
	else if(contest_state_ == "paused") contest_state = cs_paused;
	else if(contest_state_ == "after") contest_state = cs_after;
	else throw fatal_exception("server: Backup contains illegal server state.");
	
	elapsed_seconds = elapsed_seconds_;

	// counting submissions
	for(;;)
	{
		if(exists_submission(next_submission_number)) ++next_submission_number;
		else break;		
	}
	std::cout << "Counted " << next_submission_number
		<< " previous submissions." << std::endl;
	
#ifdef CAP_CLARIFICATIONS
	// counting clarifications
	for(;;)
	{
		if(exists_clarification(next_clarification_number)) ++next_clarification_number;
		else break;		
	}
	std::cout << "Counted " << next_clarification_number
		<< " previous clarifications." << std::endl;
#endif
	
#ifdef CAP_SCOREBOARD
	// restoring the scoreboard
	{
		cached_scoreboard.clear();
		std::ifstream saved_scoreboard("scoreboard.xml",
			std::ios_base::in | std::ios_base::binary);
		if(saved_scoreboard)
		{
			char ch;
			while(saved_scoreboard.get(ch)) cached_scoreboard += ch;
			if(saved_scoreboard.bad())
			{
				cached_scoreboard.clear();
				std::cout << "Error restoring the scoreboard from the scoreboard.xml file." << std::endl;
			}
			else
			{
				std::cout << "Scoreboard restored from the scoreboard.xml file." << std::endl;
			}
		}
		else
		{
			std::cout << "No scoreboard.xml file found." << std::endl;
		}
	}
#endif

	if(contest_state == cs_running)
		seconds_into_this_minute = time(NULL) % 60;
}

void internal_state::write_scoreboard()
{
	cached_scoreboard = create_scoreboard_string();
	if(!update_scoreboard_file(cached_scoreboard))
		throw fatal_exception("server: Writing the scoreboard failed.");
	std::cout << "Scoreboard updated." << std::endl;
}

/**
 * \brief Compute new #elapsed_seconds and return it
 *
 * This function computes a new value for #elapsed_seconds using
 * #contest_state and #seconds_into_this_minute.
 *
 * \return the seconds elapsed since contest start
 */
int internal_state::get_time()
{
	if(contest_state == cs_running)
	{
		int new_seconds_into_this_minute = time(NULL) % 60;

		int new_elapsed_seconds = new_seconds_into_this_minute
			- seconds_into_this_minute;
		if(new_elapsed_seconds < 0) new_elapsed_seconds += 60;
	
		int old_elapsed_seconds = elapsed_seconds;		
		if(new_elapsed_seconds > 0)
		{
			elapsed_seconds += new_elapsed_seconds;
			seconds_into_this_minute = new_seconds_into_this_minute;
		}
		
		if(auto_finish
			&& (old_elapsed_seconds / 60 < duration)
			&& (elapsed_seconds / 60 >= duration))
			contest_state = cs_after;
	}

	autosave_server_state_backup();
	autoupdate_scoreboard();
		
	return elapsed_seconds;
}

/**
 * \brief Return backup data in a string.
 *
 * This function computes a string containing the data to be written
 * in the backup file.
 *
 * \return the backup data
 */
std::string internal_state::create_server_state_backup()
{
	std::ostringstream backup_data;
	backup_data << "Overlord Server state backup file\n";
	switch(contest_state)
	{
		case cs_before: backup_data << "before\n"; break;
		case cs_running: backup_data << "running\n"; break;
		case cs_paused: backup_data << "paused\n"; break;
		case cs_after: backup_data << "after\n"; break;
	}
	backup_data << elapsed_seconds << "\n";

	return backup_data.str() + configfile;
}

/**
 * \brief Automatically update the server state
 *
 * This function gets called whenever get_time() is called and writes
 * contest state changes immediately and elapsed time every minute
 * to the backup file.
 *
 * If CAP_AUTOBACKUP is not defined this function has no effect.
 *
 * \see CAP_AUTOBACKUP
 */
void internal_state::autosave_server_state_backup()
{
#ifdef CAP_AUTOBACKUP
	static bool first = true;
	static int old_time = 0;
	static cs_t old_state = cs_before;
	
	if(first || old_time != elapsed_seconds / 60 || old_state != contest_state)
	{
		first = false;
		old_time = elapsed_seconds / 60;
		old_state = contest_state;
		
		if(save_server_state_backup(create_server_state_backup()))
			std::cout << "Autobackup for minute " << old_time << " done." << std::endl;
		else
			std::cout << "Autobackup for minute " << old_time << " failed." << std::endl;
	}
#endif
}

/**
 * \brief Return the scoreboard in a string.
 *
 * This function computes a string containing the XML scoreboard.
 *
 * If CAP_SCOREBOARD is not defined this function returns an empty string.
 *
 * \return the scoreboard
 *
 * \see CAP_SCOREBOARD
 */
std::string internal_state::create_scoreboard_string() const
{
#ifdef CAP_SCOREBOARD
	scoreboard::contest c;
	c.description = description;
	c.ctime = elapsed_seconds / 60;
	c.duration = duration;

	std::vector<scoreboard::problem> P;
	for(problem_map::const_iterator i = problems.begin(); i != problems.end(); ++i)
	{
		scoreboard::problem p;
		p.id = i->first;
		p.name = i->second;

		P.push_back(p);
	}

	std::vector<scoreboard::team> T;
	for(account_map::const_iterator i = accounts.begin(); i != accounts.end(); ++i)
	{
		if(!i->second->is_contestant()) continue;
		if(i->second->is_hidden()) continue;

		scoreboard::team t;
		t.id = i->first;
		t.name = i->second->get_name();

		T.push_back(t);
	}

	std::vector<scoreboard::submission> S;
	for(int i = 0; i != next_submission_number; ++i)
	{
		std::string submitter;
		int stime;
		std::string problem_id, language_id;
		get_submission_info(i, submitter, stime,
			problem_id, language_id);
		std::string state, reason, judge, lock;
		get_submission_state(i, state, reason, judge, lock,
#ifdef CAP_NOTIFIES
			".jnotified"
#else
			".judgement"
#endif
				);

		scoreboard::submission s;
		s.submitter = submitter;
		s.problem_id = problem_id;
		s.state = state;
		s.stime = stime;

		S.push_back(s);
	}

	return create_scoreboard(c, P, T, S, penalty);
#else
	return "";
#endif
}

/**
 * \brief Update the scoreboard every minute until the scoreboard duration is reached.
 *
 * This function gets called whenever get_time() is called and writes
 * the current scoreboard every minute to a file 'scoreboard.xml' and
 * replaces the cached_scoreboard string which is sent upon scoreboard
 * requests.
 *
 * If CAP_SCOREBOARD is not defined this function has no effect.
 *
 * \see CAP_SCOREBOARD
 */
void internal_state::autoupdate_scoreboard()
{
#ifdef CAP_SCOREBOARD
	static bool first = true;
	static int old_time = 0;
	
	if(first || old_time != elapsed_seconds / 60)
	{
		first = false;
		old_time = elapsed_seconds / 60;
		
		if(old_time <= scoreboard_duration)
		{
			cached_scoreboard = create_scoreboard_string();
			if(update_scoreboard_file(cached_scoreboard))
				std::cout << "Writing scoreboard for minute " << old_time << " done." << std::endl;
			else
				std::cout << "Writing scoreboard for minute " << old_time << " failed." << std::endl;
		}
	}
#endif
}

/**
 * \brief Return the server status in a string.
 *
 * This function computes a string containing the XML server status.
 * The contest state, time in seconds, number of received submissions and
 * clarifications is included. The structure also contains the compiletime
 * configuration, i.e. version and capabilities, and the runtime configuration,
 * i.e. the name of the configuration file, the contest name, duration,
 * scoreboard duration, penalty, programming languages and problemset,
 * maximum message size, the heartbeat interval, connection timeout and
 * automatic finish configuration. The accounts and established connections
 * are als enumerated as well as the new connections.
 * Some of the information may be missing if the capability it depends on
 * is disabled.
 *
 * If CAP_SERVERSTATUS is not defined this function returns an empty string.
 *
 * \return the server status
 *
 * \see CAP_SERVERSTATUS
 */
std::string internal_state::create_serverstatus_string() const
{
#ifdef CAP_SERVERSTATUS
	std::ostringstream status;
	
	status
		<< "<?xml version=\"1.0\" encoding=\"UTF-8\" ?>\n"
		<< "<!DOCTYPE serverstatus>\n"
		<< "<serverstatus>\n"
		<< "\t<contest-state>\n"
		<< "\t\t<state>";
	if(contest_state == cs_before) status << "before";
	else if(contest_state == cs_running) status << "running";
	else if(contest_state == cs_paused) status << "paused";
	else if(contest_state == cs_after) status << "after";
	else status << "unknown";
	status
		<< "</state>\n"
		<< "\t\t<time-seconds>" << elapsed_seconds << "</time-seconds>\n"
		<< "\t\t<time>" << elapsed_seconds / 60 << "</time>\n"
		<< "\t\t<submissions>" << next_submission_number << "</submissions>\n"
#ifdef CAP_CLARIFICATIONS
		<< "\t\t<clarifications>" << next_clarification_number << "</clarifications>\n"
#endif
		<< "\t</contest-state>\n"
		
		<< "\t<compiletime-config>\n"
		<< "\t\t<version>" << escape_xml_specials(CAP_VERSION) << "</version>\n"
#ifdef CAP_CR_TOLERANT
		<< "\t\t<cap>CAP_CR_TOLERANT</cap>\n"
#endif
#ifdef CAP_STATUS
		<< "\t\t<cap>CAP_STATUS</cap>\n"
#endif
#ifdef CAP_AUTOCONFIG
		<< "\t\t<cap>CAP_AUTOCONFIG</cap>\n"
#endif
#ifdef CAP_AUTOBACKUP
		<< "\t\t<cap>CAP_AUTOBACKUP</cap>\n"
#endif
#ifdef CAP_SCOREBOARD
		<< "\t\t<cap>CAP_SCOREBOARD</cap>\n"
#endif
#ifdef CAP_NOTIFIES
		<< "\t\t<cap>CAP_NOTIFIES</cap>\n"
#endif
#ifdef CAP_PAUSE
		<< "\t\t<cap>CAP_PAUSE</cap>\n"
#endif
#ifdef CAP_CLARIFICATIONS
		<< "\t\t<cap>CAP_CLARIFICATIONS</cap>\n"
#endif
#ifdef CAP_SERVERSTATUS
		<< "\t\t<cap>CAP_SERVERSTATUS</cap>\n"
#endif
#ifdef CAP_MAX_MESSAGE_SIZE
		<< "\t\t<cap>CAP_MAX_MESSAGE_SIZE</cap>\n"
#endif
#ifdef CAP_CONNECTION_TIMEOUT
		<< "\t\t<cap>CAP_CONNECTION_TIMEOUT</cap>\n"
#endif
		<< "\t</compiletime-config>\n"
		
		<< "\t<runtime-config>\n"
		<< "\t\t<source-file>" << escape_xml_specials(configfile) << "</source-file>\n"
		<< "\t\t<name>" << escape_xml_specials(description) << "</name>\n"
		<< "\t\t<duration>" << duration << "</duration>\n"
#ifdef CAP_SCOREBOARD
		<< "\t\t<scoreboard-duration>" << scoreboard_duration << "</scoreboard-duration>\n"
		<< "\t\t<penalty>" << penalty << "</penalty>\n"
#endif

		<< "\t\t<languages>\n";
	for(language_map::const_iterator i = languages.begin(); i != languages.end(); ++i)
		status
			<< "\t\t\t<language>\n"
			<< "\t\t\t\t<id>" << escape_xml_specials(i->first) << "</id>\n"
			<< "\t\t\t\t<name>" << escape_xml_specials(i->second) << "</name>\n"
			<< "\t\t\t</language>\n";	
	status
		<< "\t\t</languages>\n"
		
		<< "\t\t<problems>\n";
	for(problem_map::const_iterator i = problems.begin(); i != problems.end(); ++i)
		status
			<< "\t\t\t<problem>\n"
			<< "\t\t\t\t<id>" << escape_xml_specials(i->first) << "</id>\n"
			<< "\t\t\t\t<name>" << escape_xml_specials(i->second) << "</name>\n"
			<< "\t\t\t</problem>\n";	
	status
		<< "\t\t</problems>\n"
		
#ifdef CAP_MAX_MESSAGE_SIZE
		<< "\t\t<max-message-size>" << max_message_size << "</max-message-size>\n"
#endif
#ifdef CAP_CONNECTION_TIMEOUT
		<< "\t\t<heartbeat-interval>" << heartbeat_interval << "</heartbeat-interval>\n"
		<< "\t\t<connection-timeout>" << connection_timeout << "</connection-timeout>\n"
#endif

		<< "\t\t<auto-finish>" << int(auto_finish) << "</auto-finish>\n"
		<< "\t</runtime-config>\n"
		
		<< "\t<accounts>\n";
	for(account_map::const_iterator i = accounts.begin(); i != accounts.end(); ++i)
		status
			<< "\t\t<account>\n"
			<< "\t\t\t<login>" << escape_xml_specials(i->first) << "</login>\n"
			<< "\t\t\t<flags>" << escape_xml_specials(i->second->get_flags_string()) << "</flags>\n"
			<< "\t\t\t<name>" << escape_xml_specials(i->second->get_name()) << "</name>\n"
			<< "\t\t</account>\n";
	status
		<< "\t</accounts>\n"
			
		<< "\t<connections>\n";
	for(new_connection_list::const_iterator i = new_connections.begin(); i != new_connections.end(); ++i)
		status << "\t\t<new/>\n";
	for(connection_map::const_iterator i = connections.begin(); i != connections.end(); ++i)
		status
			<< "\t\t<established>\n"
			<< "\t\t\t<login>" << escape_xml_specials(i->first) << "</login>\n"
			<< "\t\t\t<flags>" << escape_xml_specials(i->second->get_account().get_flags_string()) << "</flags>\n"
			<< "\t\t\t<name>" << escape_xml_specials(i->second->get_account().get_name()) << "</name>\n"
			<< "\t\t\t<unanswered-heartbeats>" << i->second->get_unanswered_heartbeats() << "</unanswered-heartbeats>\n"
			<< "\t\t\t<locks>" << i->second->get_locks_number() << "</locks>\n"
			<< "\t\t</established>\n";
	status
		<< "\t</connections>\n"
		
		<< "</serverstatus>\n";

/*
		new_connection_list new_connections;	///< a list of all connections not yet logged in (without an account)
		connection_map connections;	///< a list of all connections that are logged in (with an account)
*/

	return status.str();
#else
	return "";
#endif
}

/**
 * \brief Notify contestants about changed submission states
 *
 * Checks all submissions and sends a notification to the corresponding
 * contestant when the state has changed. The notification is delayed
 * when a contestant is not logged in.
 *
 * If CAP_NOTIFIES is not defined this function has no effect.
 *
 * \see CAP_NOTIFIES
 */
void internal_state::notify_contestants()
{
#ifdef CAP_NOTIFIES
	for(int i = 0; i != next_submission_number; ++i)
	{
		// query the infos
		std::string submitter;
		int stime;
		std::string problem_id, language_id;
		get_submission_info(i, submitter, stime,
			problem_id, language_id);

		// is the contestant logged in?
		connection_map::iterator j = connections.find(submitter);
		if(j == connections.end()) continue;	// _not_ logged in
		if(!need_submission_notification(i))
			continue;	// does _not_ need notification
		
		std::string state, reason, judge, lock;
		get_submission_state(i, state, reason, judge, lock, ".notified");
		
		std::ostringstream msg;
		msg << "submission_result\n"
			<< i << "\n"
			<< stime << "\n"
			<< problem_id << "\n"
			<< language_id << "\n"
			<< "notifies\n"
			<< state << "\n"
			<< reason << "\n";
		j->second->get_socket().send(msg.str());
	}
#endif
}

/**
 * \brief Send submission list to a contestant
 *
 * This function sends the current list of the submissions as seen
 * by a contestant with the given login to a socket.
 *
 * \param login the login of the contestant
 * \param socket the socket to write to
 */
void internal_state::send_submission_results(const std::string &login,
	socket_interface &socket)
{
	for(int i = 0; i != next_submission_number; ++i)
	{
		// query the infos
		std::string submitter;
		int stime;
		std::string problem_id, language_id;
		get_submission_info(i, submitter, stime,
			problem_id, language_id);

		// only show submissions by this contestant
		if(submitter == login)
		{
			std::string state, reason, judge, lock;
#ifdef CAP_NOTIFIES
			bool notified = need_submission_notification(i);
#endif
			get_submission_state(i, state, reason, judge, lock,
#ifdef CAP_NOTIFIES
				".notified"
#else
				".judgement"
#endif
				);
			
			std::ostringstream answer;
			answer << "submission_result\n"
				<< i << "\n"
				<< stime << "\n"
				<< problem_id << "\n"
				<< language_id << "\n"
#ifdef CAP_NOTIFIES
				<< (notified ? "notifies\n" : "\n")
#else
				<< "\n"
#endif
				<< state << "\n"
				<< reason << "\n";
			socket.send(answer.str());
		}
	}
}

/**
 * \brief Send submission list to a judge
 *
 * This function sends the current list of all submissions as seen
 * by the judges to a socket.
 *
 * \param socket the socket to write to
 */
void internal_state::send_submission_list(socket_interface &socket)
{
	for(int i = 0; i != next_submission_number; ++i)
	{
		// query the infos
		std::string submitter;
		int stime;
		std::string problem_id, language_id;
		get_submission_info(i, submitter, stime,
			problem_id, language_id);

		std::string state, reason, judge, lock;
		get_submission_state(i, state, reason, judge, lock,
#ifdef CAP_NOTIFIES
			".jnotified"
#else
			".judgement"
#endif
			);

		std::ostringstream answer;
		answer << "submission_notify\n"
			<< i << "\n"
			<< submitter << "\n"
			<< stime << "\n"
			<< problem_id << "\n"
			<< language_id << "\n"
			<< "\n"
			<< judge << "\n"
			<< state << "\n"
			<< reason << "\n"
			<< lock << "\n";
		socket.send(answer.str());
	}
}

/**
 * \brief Notify judges about changed submission states
 *
 * Checks all submissions and sends notifications when the state
 * has changed. Notifications are send to all judges that are logged
 * in.
 *
 * If CAP_NOTIFIES is not defined this function has no effect.
 *
 * \see CAP_NOTIFIES
 */
void internal_state::notify_judges()
{
#ifdef CAP_NOTIFIES
	for(int i = 0; i != next_submission_number; ++i)
	{
		if(!need_submission_notification(i, true))
			continue;	// already notified
		
		// query the infos
		std::string submitter;
		int stime;
		std::string problem_id, language_id;
		get_submission_info(i, submitter, stime,
			problem_id, language_id);

		std::string state, reason, judge, lock;
		get_submission_state(i, state, reason, judge, lock, ".jnotified");

		std::ostringstream msg;
		msg << "submission_notify\n"
			<< i << "\n"
			<< submitter << "\n"
			<< stime << "\n"
			<< problem_id << "\n"
			<< language_id << "\n"
			<< "notifies\n"
			<< judge << "\n"
			<< state << "\n"
			<< reason << "\n"
			<< lock << "\n";

		for(connection_map::iterator j = connections.begin();
			j != connections.end(); ++j)
		{
			if(j->second->get_account().is_judge())
				j->second->get_socket().send(msg.str());
		}
	}
#endif
}

/**
 * \brief Send submission list to a client
 *
 * This function sends the current list of clarification requests as seen
 * by a specific client to a socket.
 *
 * If CAP_CLARIFICATIONS is not defined this function has no effect.
 *
 * \param login the login of the client
 * \param socket the socket to write to
 *
 * \see CAP_CLARIFICATIONS
 */
void internal_state::send_clarification_replies(const std::string &login,
	socket_interface &socket)
{
#ifdef CAP_CLARIFICATIONS
	for(int i = 0; i != next_clarification_number; ++i)
	{
		// query the infos
		std::string submitter, problem, question;
		int stime;
		get_clarification_info(i, submitter, problem,
			stime, question);

		std::string state, ctime, judge, lock, answer;
		bool own_question = (submitter == login);
#ifdef CAP_NOTIFIES
		bool state_change;
		bool notified = own_question && need_clarification_notification(i, state_change);
#endif
		get_clarification_state(i, state, ctime, judge, lock, answer,
#ifdef CAP_NOTIFIES
			(own_question ? ".notified" : ".onotified")
#else
			".answer"
#endif
			);
				
		// only show questions by this client and answers to everyone
		if(own_question || state == "everyone")
		{
			std::ostringstream message;
			message << "clarification_reply\n"
				<< i << "\n"
				<< problem << "\n"
				<< state << "\n"
#ifdef CAP_NOTIFIES
				<< (notified ? "notifies\n" : "\n")
#else
				<< "\n"
#endif
				<< answer;
			socket.send(message.str());		
		}
	}
#endif
}

/**
 * \brief Send clarification list to a judge
 *
 * This function sends the current list of all clarification requests as seen
 * by the judges to a socket.
 *
 * If CAP_CLARIFICATIONS is not defined this function has no effect.
 *
 * \param socket the socket to write to
 *
 * \see CAP_CLARIFICATIONS
 */
void internal_state::send_clarification_list(socket_interface &socket)
{
#ifdef CAP_CLARIFICATIONS
	for(int i = 0; i != next_clarification_number; ++i)
	{
		// query the infos
		std::string submitter, problem, question;
		int stime;
		get_clarification_info(i, submitter, problem,
			stime, question);

		std::string state, ctime, judge, lock, answer;
		get_clarification_state(i, state, ctime, judge, lock, answer,
#ifdef CAP_NOTIFIES
			".onotified"
#else
			".answer"
#endif
			);

		std::ostringstream message;
		message << "clarification_notify\n"
			<< i << "\n"
			<< submitter << "\n"
			<< problem << "\n"
			<< stime << "\n"
			<< question;
		socket.send(message.str());
				
		std::ostringstream message2;
		message2 << "clarification_notify2\n"
			<< i << "\n"
			<< judge << "\n"
			<< state << "\n"
			<< ctime << "\n"
			<< "\n"
			<< lock << "\n"
			<< answer;
		socket.send(message2.str());
	}
#endif
}

/**
 * \brief Notify questioners about changed clarification answers
 *
 * Checks all clarification requests and sends a notification to the
 * questioner when the state has changed and the lock string is empty.
 * The notification is delayed when a questioner is not logged in.
 *
 * If CAP_CLARIFICATIONS or CAP_NOTIFIES is not defined this function
 * has no effect.
 *
 * \see CAP_CLARIFICATIONS and CAP_NOTIFIES
 */
void internal_state::notify_clarifications_questioner()
{
#ifdef CAP_CLARIFICATIONS
#ifdef CAP_NOTIFIES
	for(int i = 0; i != next_clarification_number; ++i)
	{
		// query the infos
		std::string submitter, problem, question;
		int stime;
		get_clarification_info(i, submitter, problem,
			stime, question);
		
		// is the questioner logged in?
		connection_map::iterator j = connections.find(submitter);
		if(j == connections.end()) continue;	// _not_ logged in
		bool state_change;
		if(!need_clarification_notification(i, state_change))
			continue;	// does _not_ need notification
		
		std::string state, ctime, judge, lock, answer;
		get_clarification_state(i, state, ctime, judge, lock, answer, ".notified");
		
		std::ostringstream message;
		message << "clarification_reply\n"
			<< i << "\n"
			<< problem << "\n"
			<< state << "\n"
			<< "notifies\n"
			<< answer;
		j->second->get_socket().send(message.str());
	}
#endif
#endif
}

/**
 * \brief Notify others about changed clarification states
 *
 * Checks all clarification requests and sends notifications when the
 * state has changed. All judges that are currently logged in are
 * notified and if the answer is for everyone and the lock string is
 * empty, every client that is logged in receives a notification as well.
 *
 * If CAP_CLARIFICATIONS or CAP_NOTIFIES is not defined this function
 * has no effect.
 *
 * \see CAP_CLARIFICATIONS and CAP_NOTIFIES
 */
void internal_state::notify_clarifications_others()
{
#ifdef CAP_CLARIFICATIONS
#ifdef CAP_NOTIFIES
	for(int i = 0; i != next_clarification_number; ++i)
	{
		bool state_change;
		if(!need_clarification_notification(i, state_change, true))
			continue;	// already notified
		
		// query the infos
		std::string submitter, problem, question;
		int stime;
		get_clarification_info(i, submitter, problem,
			stime, question);
		
		std::string state, ctime, judge, lock, answer;
		get_clarification_state(i, state, ctime, judge, lock, answer, ".onotified");

		// send notification to the judges
		std::ostringstream message;
		message << "clarification_notify\n"
			<< i << "\n"
			<< submitter << "\n"
			<< problem << "\n"
			<< stime << "\n"
			<< question;
				
		std::ostringstream message2;
		message2 << "clarification_notify2\n"
			<< i << "\n"
			<< judge << "\n"
			<< state << "\n"
			<< ctime << "\n"
			<< "notifies\n"
			<< lock << "\n"
			<< answer;
		
		for(connection_map::iterator j = connections.begin();
			j != connections.end(); ++j)
		{
			if(j->second->get_account().is_judge())
			{
				j->second->get_socket().send(message.str());
				j->second->get_socket().send(message2.str());
			}
		}
		
		// possibly send notification to everyone
		if(!state_change)	// only the lock was notified...
			continue;	//  ...do _not_ notify non-judges
		if(state != "everyone")
			continue;	// this clarification is _not_ for everyone
		
		std::ostringstream message3;
		message3 << "clarification_reply\n"
			<< i << "\n"
			<< problem << "\n"
			<< state << "\n"
			<< "notifies\n"
			<< answer;
		
		for(connection_map::iterator j = connections.begin();
			j != connections.end(); ++j)
		{
			if(j->first != submitter)	// the questioner is notified separately
				j->second->get_socket().send(message3.str());
		}
	}
#endif
#endif
}

internal_state *internal_state::that = 0;
internal_state::sighandler_t internal_state::old_int,
	internal_state::old_term, internal_state::old_hup;

void internal_state::static_signal_int(int)
{
	that->signal_int();
}

void internal_state::static_signal_hup(int)
{
	that->signal_hup();
}

void internal_state::signal_int()
{
	terminate = true;
}

void internal_state::signal_hup()
{
	switch(contest_state)
	{
		case cs_before: change_state(cs_running); break;
		case cs_running: change_state(cs_paused); break;
		case cs_paused: change_state(cs_running); break;
		case cs_after: change_state(cs_after); break;
	}
}

void internal_state::change_state(cs_t newstate)
{
	contest_state = newstate;
	
	if(contest_state == cs_running)
		seconds_into_this_minute = time(NULL) % 60;	
}


/**
 * \brief Creates a new server object
 *
 * The function creates a new server object without any configuration.
 * All configuration functions have to be called before the server begin
 * execution.
 *
 * \return a new server object
 *
 * \see server_interface::set_configfile(), server_interface::set_description(),
 *     server_interface::set_duration(), server_interface::set_scoreboard_duration(),
 *     server_interface::set_penalty(), server_interface::add_language(),
 *     server_interface::add_problem(), server_interface::add_contestant(),
 *     server_interface::add_judge(), server_interface::set_max_message_size(),
 *     server_interface::set_connection_timeout() and server_interface::add_port()
 */
std::auto_ptr<server_interface> create_default_state()
{
	std::auto_ptr<server_interface> state(new internal_state());
	return state;
}
