/* $Id: server.hh,v 1.1 2005/07/11 21:45:39 whess Exp $ */
/* Overlord Server for the Refefree Programming Contest Judging System
 * Copyright (C) 2005 Wolfgang Hess
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
/**
 * \file server.hh
 * \brief The interface for the server object and functions to create one.
 */
#ifndef SERVER_HH_
#define SERVER_HH_ 1	///< protect against multiple inclusion

#include <memory>
#include <string>
#include <boost/utility.hpp>
#include <boost/cstdint.hpp>

/**
 * \class server_interface
 * \brief The interface of the server object.
 *
 * This is just an interface, so it is non-copyable and the
 * member functions are pure virtual.
 */
class server_interface : boost::noncopyable
{
	public:
		virtual ~server_interface() {}	///< virtual destructor, since this is a base class
	
		/**
		 * \brief The main function of the server object.
		 *
		 * This function is called when the server is fully configured. It causes
		 * the server to start and does not return until program termination.
		 *
		 * \return exit code of the program
		 */
		virtual int execute() = 0;
		
		/**
		 * \brief Configures the name of the configuration file
		 *
		 * This function informs the server about the name of its configuration
		 * file.
		 *
		 * \param configfile_ the filename of the configuration file
		 */
		virtual void set_configfile(const std::string &configfile_) = 0;
		
		/**
		 * \brief Configures the name of the contest
		 *
		 * This function sets the contest name
		 *
		 * \param description_ the name of the contest
		 */
		virtual void set_description(const std::string &description_) = 0;
		
		/**
		 * \brief Configures the duration of the contest
		 *
		 * This function sets the contest duration in minutes. This value
		 * must be positive.
		 *
		 * \param duration_ contest duration in minutes
		 */
		virtual void set_duration(int duration_) = 0;
		
		/**
		 * \brief Configures the duration of the scoreboard
		 *
		 * This function sets the time the scoreboard will be shown in minutes
		 * since contest start. This value must be non-negative.
		 *
		 * \param scoreboard_duration_ the duration for which the scoreboard
		 *     will be shown
		 */
		virtual void set_scoreboard_duration(int scoreboard_duration_) = 0;
		
		/**
		 * \brief Configures the penalty for wrong submissions
		 *
		 * This function sets the penalty for wrong submissions in minutes.
		 * The value must be non-negative.
		 *
		 * \param penalty_ the penalty in minutes per wrong submission
		 */
		virtual void set_penalty(int penalty_) = 0;
		
		/**
		 * \brief Configures the maximal size for messages
		 *
		 * This function sets the maximal size of accepted messages. It only
		 * needs to be called if CAP_MAX_MESSAGE_SIZE is defined. The value
		 * must be a positive integer.
		 *
		 * \param max_message_size_ the maximal message size in kilobytes
		 *
		 * \see CAP_MAX_MESSAGE_SIZE
		 */
		virtual void set_max_message_size(int max_message_size_) = 0;
		
		/**
		 * \brief Configures the heartbeat interval
		 *
		 * This function sets the heartbeat interval. It only needs to be
		 * called if CAP_CONNECTION_TIMEOUT is defined. The value must
		 * be a positive integer, which is the heartbeat interval in seconds
		 * between two heartbeats.
		 *
		 * \param heartbeat_interval_ the heartbeat interval in seconds
		 *
		 * \see CAP_CONNECTION_TIMEOUT
		 */
		virtual void set_heartbeat_interval(int heartbeat_interval_) = 0;
		
		/**
		 * \brief Configures the connection timeout
		 *
		 * This function sets the connection timeout. It only needs to be
		 * called if CAP_CONNECTION_TIMEOUT is defined. The value must either
		 * be a positive integer, or zero to disable the mechanism.
		 *
		 * \param connection_timeout_ the connection timeout in heartbeats
		 *
		 * \see CAP_CONNECTION_TIMEOUT
		 */
		virtual void set_connection_timeout(int connection_timeout_) = 0;
		
		/**
		 * \brief Configures the auto-finish
		 *
		 * This function enables or disables the auto-finish. If enabled,
		 * the server automatically changes the contest state to 'after'
		 * when the contest time is over.
		 *
		 * \param auto_finish_ true if auto-finish is enabled, false if it
		 * is disabled
		 */
		virtual void set_auto_finish(bool auto_finish_) = 0;
		
		/**
		 * \brief Opens a port to which the server is listening
		 *
		 * This function opens a port to accept client logins. If the port
		 * cannot be openend a fatal_exception is thrown.
		 *
		 * \param port number of the port to listen to
		 * \param accept_contestants determines whether contestants are accepted
		 *     at this port
		 * \param accept_judges determines whether judges are accepted at this
		 *     port
		 * \param use_local determines if this port only binds to 127.0.0.1 or
         *     to all addresses
		 */
		virtual void add_port(boost::uint16_t port, bool accept_contestants,
			bool accept_judges, bool use_local) = 0;
		
		/**
		 * \brief Adds a judge account to the server.
		 *
		 * Throws a fatal_exception if the login exists already.
		 *
		 * \param login login of the new account
		 * \param passwd password of the new account
		 */
		virtual void add_judge(const std::string &login,
			const std::string &passwd) = 0;
			
		/**
		 * \brief Adds a contestant account to the server.
		 *
		 * Throws a fatal_exception if the login exists already.
		 *
		 * \param login login of the new account
		 * \param passwd password of the new account
		 * \param name the full name of the team
		 * \param hide whether to hide this contestant account from the scoreboard
		 */
		virtual void add_contestant(const std::string &login,
			const std::string &passwd, const std::string &name,
			bool hide = false) = 0;
			
		/**
		 * \brief Adds a programming language to the server.
		 *
		 * Throws a fatal_exception if the language id exists already.
		 *
		 * \param id id of the new language
		 * \param name the full name of the programming language
		 */
		virtual void add_language(const std::string &id,
			const std::string &name) = 0;
			
		/**
		 * \brief Adds a problem to the problem set of the server.
		 *
		 * Throws a fatal_exception if the problem id exists already.
		 * 
		 * \param id id of the new problem
		 * \param name the full name of the problem
		 */
		virtual void add_problem(const std::string &id,
			const std::string &name) = 0;
		
		/**
		 * \brief Restores a contest from backup information
		 * 
		 * This function sets the contest state and elapsed time and
		 * searches for submission and clarification files.
		 *
		 * \param contest_state_ the contest state fromt the backup, either
		 *     before, running, paused or after
		 * \param elapsed_seconds_ the time in seconds since contest start
		 */
		virtual void restore_backup(const std::string &contest_state_,
			int elapsed_seconds_) = 0;
			
		/**
		 * \brief Write a current scoreboard.xml file.
		 * 
		 * This function writes a current scoreboard to the scoreboard.xml
		 * file. This function is intended to write the final scoreboard
		 * once the contest is over.
		 */
		 virtual void write_scoreboard() = 0;
};

std::auto_ptr<server_interface> create_default_state();

// implemented in config.cc
std::auto_ptr<server_interface> state_from_config(const std::string &config, bool open_ports = true);
std::auto_ptr<server_interface> state_from_backup(bool open_ports = true);

#endif // SERVER_HH_
