/* $Id: socket.cc,v 1.1 2005/07/11 21:45:39 whess Exp $ */
/* Overlord Server for the Refefree Programming Contest Judging System
 * Copyright (C) 2005 Wolfgang Hess
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
/**
 * \file socket.cc
 * \brief Implements TCP sockets.
 */

#include <memory>
#include <sstream>
#include <string>
#include <deque>
#include <algorithm>
#include <iostream>
#include <iomanip>
#include <cerrno>
#include <boost/utility.hpp>
#include <boost/cstdint.hpp>

#include "socket.hh"
#include "exception.hh"

#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netinet/tcp.h>
#include <fcntl.h>
#include <unistd.h>

namespace
{
	/// set socketfd to non-blocking and disable Nagle for lower latency
	void setup_socket_opts(int socketfd)
	{
		// set non-blocking mode
		int flags = fcntl(socketfd, F_GETFL, 0);
		if(flags == -1) throw fatal_exception("net: fcntl for non-blocking socket failed. ["
			+ std::string(strerror(errno)) + "]");
		flags |= O_NONBLOCK;
		if(fcntl(socketfd, F_SETFL, flags) == -1)
			throw fatal_exception("net: fcntl for non-blocking socket failed. ["
				+ std::string(strerror(errno)) + "]");

		// disable Nagle
		int flag = 1;
		setsockopt(socketfd, IPPROTO_TCP, TCP_NODELAY, reinterpret_cast<char *>(&flag), sizeof(int));
	}
}

/**
 * \class tcp_socket
 * \brief Implements a TCP socket
 * 
 */
class tcp_socket : public socket_interface
{
	public:
		tcp_socket(int s_);
		~tcp_socket();
		
		bool process();
		void send(const std::string &out);
		bool message_too_large(int size_limit_kb);
		bool receive(std::string &in);
		std::string get_debug_string();
			
	private:
		int s;	///< the (owned) socket descriptor
		bool eof;	///< did we receive eof?
		bool reset;	///< we received ECONNRESET
	
		std::deque<char> in_buffer;	///< data that has been received but not used
		std::deque<char> out_buffer;	///< data yet to be sent
		
		boost::uint32_t snoop_header();
		
		std::string peername;	///< name of the peer to return by get_debug_string()
};

/**
 * \brief Create an object from a socket descriptor
 *
 * This function assumes that the socket can be taken as is and
 * is non-blocking. This function takes the ownership of the 
 * descriptor. It is used by tcp_server_socket::accept() and
 * should not be used directly.
 *
 * \param s_ the existing socket descriptor to use
 */
tcp_socket::tcp_socket(int s_)
	: s(s_), eof(false), reset(false), peername("<unknown>")
{
	sockaddr_in addr;
	socklen_t size = sizeof(addr);
	if(getpeername(s, (sockaddr *)&addr, &size) == 0
		&& size == sizeof(addr) && addr.sin_family == AF_INET)
	{
		const uint32_t ipaddr = ntohl(addr.sin_addr.s_addr);
		std::ostringstream os;
		os << ((ipaddr >> 24) & 0xff) << "."
			<< ((ipaddr >> 16) & 0xff) << "."
			<< ((ipaddr >> 8) & 0xff) << "."
			<< (ipaddr & 0xff);
		peername = os.str();
	}
}

tcp_socket::~tcp_socket()
{
	close(s);
}

bool tcp_socket::process()
{
	bool fail = false;
	char sendbuf[2048], *sb_rdpos = sendbuf, *sb_wrpos = sendbuf;
	char * const sb_begin = sendbuf, * const sb_end = sb_begin + sizeof(sendbuf);

	if(!reset) do
	{
		for(; sb_end - sb_wrpos > 0 && out_buffer.size() > size_t(sb_wrpos - sb_rdpos);)
		{	
			*sb_wrpos = out_buffer[sb_wrpos - sb_rdpos];
			++sb_wrpos;
		}

		if(sb_rdpos < sb_wrpos)	// try to send
		{
			int size = write(s, sb_rdpos, sb_wrpos - sb_rdpos);
			if(size == -1)
			{
				if(errno == ECONNRESET) reset = true;
				fail = true;
			}
			else
			{
				sb_rdpos += size;
				out_buffer.erase(out_buffer.begin(), out_buffer.begin() + size);
				while(size_t(sb_rdpos - sb_begin) >= sizeof(sendbuf) / 2)
				{
					memcpy(sb_begin, sb_rdpos, sb_wrpos - sb_rdpos);
					sb_wrpos -= (sb_rdpos - sb_begin);
					sb_rdpos = sb_begin;
				}
			}
		}
		else fail = true;
	} while(!fail);
	
	if(eof) return false;
		
	// try to receive
	char recvbuf[1024];
	ssize_t size = read(s, recvbuf, sizeof(recvbuf));
	while(size != -1 && size > 0)
	{
		copy(recvbuf + 0, recvbuf + size, std::back_inserter<std::deque<char> >(in_buffer));
		size = read(s, recvbuf, sizeof(recvbuf));
	}
			
	if(size == 0) eof = true;	// eof received

	return true;
}

void tcp_socket::send(const std::string &out)
{
	std::string::size_type size = out.size();
	if((size >> 24) > 255) throw fatal_exception("net: Unable to send blocks larger than 4GB.");
	
	// send the string size as a decimal integer in a 10 character, space padded field
	std::ostringstream size_s;
	size_s << std::left <<  std::setfill(' ') << std::setw(10) << size;
	const std::string size_s2 = size_s.str();
	out_buffer.insert(out_buffer.end(), size_s2.begin(), size_s2.end());

	// send the string
	out_buffer.insert(out_buffer.end(), out.begin(), out.end());
}

bool tcp_socket::message_too_large(int size_limit_kb)
{
	if(in_buffer.size() < 10) return false;
	boost::uint32_t size = snoop_header();
	if(size == 0) return false;
	return ((size - 1) >> 10) >= unsigned(size_limit_kb);
}

bool tcp_socket::receive(std::string &in)
{
	if(in_buffer.size() < 10) return false;	// not even a block header
	boost::uint32_t size = snoop_header();	
	if(in_buffer.size() - 10 < size) return false;	// block not completely received
	
	// move the string out of the receive buffer
	in.clear();
	
	// changed to circumvent a bug in MacOS X 10.2 GCC
	// in.insert(in.begin(), in_buffer.begin() + 10, in_buffer.begin() + 10 + size);
	std::copy(in_buffer.begin() + 10, in_buffer.begin() + 10 + size, std::back_inserter(in));
	
	in_buffer.erase(in_buffer.begin(), in_buffer.begin() + 10 + size);

	return true;
}

std::string tcp_socket::get_debug_string()
{
	return peername;	
}

/**
 * \brief Returns the size of the next message
 *
 * This function looks at the first 10 bytes of the input queue to determine
 * the size of the next message. If the message header is malformed
 * a protocol_exception is thrown.
 *
 * When this function is called there must be at least 10 bytes in the input
 * queue. This function does not remove data from the queue.
 *
 * \return the size of the next message body in bytes
 *
 * \see receive(), message_too_large()
 */
boost::uint32_t tcp_socket::snoop_header()
{
	// receive the string size as a decimal integer in a 10 character, space padded field

	// changed to circumvent a bug in MacOS X 10.2 GCC
	// std::string size_s2(in_buffer.begin(), in_buffer.begin() + 10);
	std::string size_s2;
	std::copy(in_buffer.begin(), in_buffer.begin() + 10, std::back_inserter(size_s2));
	
	boost::uint32_t size = 0;
	try	// check the string for correct format "0|([1-9][0-9]* *)"
	{
		if(size_s2[0] == '0')
		{
			if(size_s2.find_first_not_of(" ", 1) != std::string::npos) throw 0;
		}
		else
		{
			std::string::size_type i = size_s2.find_first_not_of("0123456789", 0);
			if(i == 0) throw 0;
			if(size_s2.find_first_not_of(" ", i) != std::string::npos) throw 0;
		}
		std::istringstream size_s(size_s2);
		if(!(size_s >> size)) throw 0;
	}
	catch(int)
	{
		throw protocol_exception("net: Size header malformed");
	}
	
	return size;
}

/**
 * \class tcp_server_socket
 * \brief Implements a listening TCP (server) socket.
 */
class tcp_server_socket : public server_socket_interface
{
	public:
		tcp_server_socket(boost::uint16_t port, bool use_local);
		~tcp_server_socket();
		std::auto_ptr<socket_interface> accept();
		
	private:
		int s;	///< the socket descriptor
};

/**
 * \brief Create a new server socket for a given port
 *
 * The constructor opens a non-blocking socket for listening.
 *
 * \param port the port to listen to
 * \param use_local whether to only bind to 127.0.0.1, or all addresses
 */
tcp_server_socket::tcp_server_socket(boost::uint16_t port, bool use_local)
	: s(-1)
{
	const int listen_queue_size = 16;
		
	// try to open a new server socket
	s = socket(PF_INET, SOCK_STREAM, 0);
	if(s == -1) throw fatal_exception("net: Socket creation failed. ["
		+ std::string(strerror(errno)) + "]");

	try
	{
		{
			int on = 1;
			setsockopt(s, SOL_SOCKET, SO_REUSEADDR, &on, sizeof(on));
		}
		
		sockaddr_in addr;
		addr.sin_family = AF_INET;
		addr.sin_addr.s_addr = htonl(use_local ? 0x7f000001 : INADDR_ANY);
		addr.sin_port = htons(port);
		if(bind(s, reinterpret_cast<sockaddr *>(&addr), sizeof(addr)) == -1)	// bind to port
		{
			std::ostringstream os;
			os << "net: Binding to port " << port << " failed.";
			throw fatal_exception(os.str() + " [" + std::string(strerror(errno)) + "]");
		}
		if(listen(s, listen_queue_size) == -1)	// listen to the port
			throw fatal_exception("net: Listening to bound port failed. ["
				+ std::string(strerror(errno)) + "]");

		setup_socket_opts(s);
	}
	catch(...)
	{
		close(s);
		throw;
	}
}

tcp_server_socket::~tcp_server_socket()
{
	close(s);
}

/**
 * \brief Accept a new connection.
 *
 * This function accepts a new connection if possible and returns
 * a corresponding tcp_socket. Otherwise a null pointer is returned.
 * The function does not block.
 *
 * \return a new tcp_socket or a null pointer if none exists
 */
std::auto_ptr<socket_interface> tcp_server_socket::accept()
{
	int s0;
	s0 = ::accept(s, 0, 0);

	if(s0 == -1) return std::auto_ptr<socket_interface>();
	else
	{
		setup_socket_opts(s0);
		return std::auto_ptr<socket_interface>(new tcp_socket(s0));
	}
}

/**
 * \brief Create a new server socket
 *
 * This function creates a server socket listening to a given port, either
 * at 127.0.0.1 or all addresses.
 *
 * \param port the port to listen to
 * \param use_local if true the server socket only listens to 127.0.0.1
 * \return a new server socket
 */
std::auto_ptr<server_socket_interface> create_server_socket(
	boost::uint16_t port, bool use_local)
{
	std::auto_ptr<server_socket_interface> s(new tcp_server_socket(port, use_local));
	return s;
}
