/* $Id: socket.hh,v 1.1 2005/07/11 21:45:39 whess Exp $ */
/* Overlord Server for the Refefree Programming Contest Judging System
 * Copyright (C) 2005 Wolfgang Hess
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
/**
 * \file socket.hh
 * \brief The socket interface.
 */
#ifndef SOCKET_HH_
#define SOCKET_HH_ 1	///< protect against multiple inclusion

#include <memory>
#include <string>
#include <boost/utility.hpp>
#include <boost/cstdint.hpp>

/**
 * \class socket_interface
 * \brief The socket interface for existing connections.
 *
 * This is just an interface, so it is non-copyable and the
 * member functions are pure virtual.
 */
class socket_interface : boost::noncopyable
{
	public:
		virtual ~socket_interface() {}	///< virtual destructor, since this is a base class
	
		/**
		 * \brief Send and receive data over the network.
		 *
		 * This function has to be called to actually send or receive data.
		 * The other member functions only process the deques for received data
		 * or data that is ready to be sent.
		 *
		 * This function returns false after the connection has been closed by
		 * the other side, that is no further data will be received.
		 */
		virtual bool process() = 0;
		
		/**
		 * \brief Send a message.
		 *
		 * This function adds a new message in the send queue.
		 *
		 * \param out the message to send
		 */
		virtual void send(const std::string &out) = 0;
		
		/**
		 * \brief Checks for exceedingly large messages.
		 *
		 * \param size_limit_kb the size limit in kilobytes
         * \return true, if a message with a body larger than the size limit
		 *     is at the front of the receive queue
		 */ 
		virtual bool message_too_large(int size_limit_kb) = 0;
		
		/**
		 * \brief Receive a message.
		 *
		 * This function tries to receive a message. If the receive queue
		 * contains a complete message this message is read and removed
		 * from the queue.
		 *
		 * \param in is set to the received message, or undefined otherwise
		 * \return true if a message was received, false otherwise
		 */
		virtual bool receive(std::string &in) = 0;
		
		/**
		 * \brief Output the address to a (debug) string.
		 *
		 * \return the address as a dotted quad
		 */
		virtual std::string get_debug_string() = 0;
};

/**
 * \class server_socket_interface
 * \brief The interface for listening (server) sockets.
 *
 * This is just an interface, so it is non-copyable and the
 * member functions are pure virtual.
 */
class server_socket_interface : boost::noncopyable
{
	public:
		virtual ~server_socket_interface() {}	///< virtual destructor, since this is a base class
		
		/**
		 * \brief Accept a new connection
		 *
		 * This function tries to accept a new connection. It does not block.
		 *
		 * \return a new connection or a null pointer if none exists
		 */
		virtual std::auto_ptr<socket_interface> accept() = 0;
};

std::auto_ptr<server_socket_interface> create_server_socket(
	boost::uint16_t port, bool use_local);

#endif // SOCKET_HH_
